      SUBROUTINE biology (ng,tile)
!
!svn $Id: ecodynamo.h 1368 2015-07-13 13:10:18Z mitya $Bio
!************************************************** Hernan G. Arango ***
!  Copyright (c) 2002-2011 The ROMS/TOMS Group        Craig V. Lewis   !
!    Licensed under a MIT/X style license                              !
!    See License_ROMS.txt                                              !
!***********************************************************************
!                                                                      !
!  Nutrient-Phytoplankton-Zooplankton-Detritus Model.                  !
!                                                                      !
!  This routine computes the biological sources and sinks and adds     !
!  then the global biological fields.                                  !
!                                                                      !
!  Reference:                                                          !
!                                                                      !
!    Powell, T.M., C.V.W. Lewis, E. Curchitser, D. Haidvogel,          !
!      Q. Hermann and E. Dobbins, 2006: Results from a three-          !
!      dimensional,  nested biological-physical model of the           !
!      California Current System: Comparisons with Statistics          !
!      from Satellite Imagery, J. Geophys. Res.                        !
!                                                                      !
!  Adapted from code written originally by Craig V. Lewis.             !
!                                                                      !
!***********************************************************************
!
      USE mod_param
!*8*******************Stuff imported from fennel.h**********************
#ifdef DIAGNOSTICS_BIO
      USE mod_diags
#endif
!***********************************************************************
      USE mod_forces  
      USE mod_grid
      USE mod_ncparam
      USE mod_ocean
      USE mod_stepping
      USE mod_ice
     
!
!  Imported variable declarations.
!
      integer, intent(in) :: ng, tile
!
!  Local variable declarations.
!
#include "tile.h"
!
!  Set header file name.
!
#ifdef DISTRIBUTE
      IF (Lbiofile(iNLM)) THEN
#else
      IF (Lbiofile(iNLM).and.(tile.eq.0)) THEN
#endif
        Lbiofile(iNLM)=.FALSE.
        BIONAME(iNLM)=__FILE__
      END IF
!
#ifdef PROFILE
      CALL wclock_on (ng, iNLM, 15)
#endif
      CALL biology_tile (ng, tile,                                      &
     &                   LBi, UBi, LBj, UBj, N(ng), NT(ng),             &
     &                   IminS, ImaxS, JminS, JmaxS,                    &
     &                   nstp(ng), nnew(ng),                            &
#ifdef ICE_MODEL
     &                   liold(ng), linew(ng),                          &
     &                   ICE(ng)% ai,                                   &
     &                   ICE(ng)% hi,                                   & !Confirm it is ice thickness and check units
     &                   ICE(ng)% hsn,                                  & !Confirm it is snow thickness and check units
#endif
# ifdef CICE_COUPLING
     &                   ICE(ng) % aice,                                &
# endif
#ifdef MASKING
     &                   GRID(ng) % rmask,                              &
#endif
     &                   GRID(ng) % Hz,                                 &
     &                   GRID(ng) % z_r,                                &
     &                   GRID(ng) % z_w,                                &
     &                   FORCES(ng) % srflx,                            &
!***********************Stuff imported from fennel.h****************************
#if defined CARBON || defined OXYGEN
# ifdef BULK_FLUXES
     &                   FORCES(ng) % Uwind,                            &
     &                   FORCES(ng) % Vwind,                            &
# else
     &                   FORCES(ng) % sustr,                            &
     &                   FORCES(ng) % svstr,                            &
# endif
#endif
#ifdef CARBON
     &                   OCEAN(ng) % pH,                                &
#endif
#ifdef DIAGNOSTICS_BIO
     &                   DIAGS(ng) % DiaBio2d,                          &
     &                   DIAGS(ng) % DiaBio3d,                          &
#endif
!*******************************************************************************/
     &                   OCEAN(ng) % t)

#ifdef PROFILE
      CALL wclock_off (ng, iNLM, 15)
#endif
      RETURN
      END SUBROUTINE biology
!
!-----------------------------------------------------------------------
      SUBROUTINE biology_tile (ng, tile,                                &
     &                         LBi, UBi, LBj, UBj, UBk, UBt,            &
     &                         IminS, ImaxS, JminS, JmaxS,              &
     &                         nstp, nnew,                              &
#ifdef ICE_MODEL
     &                         liold, linew,                            &
     &                          ai,                                     &
     &                          hi,                                     &
     &                          hsn,                                    &
#endif
# ifdef CICE_COUPLING
     &                          aice,                                   &
#endif
#ifdef MASKING
     &                         rmask,                                   &
#endif
     &                         Hz, z_r, z_w,                            &
     &                         srflx,                                   &
!***********************Stuff imported from fennel.h****************************
#if defined CARBON || defined OXYGEN
# ifdef BULK_FLUXES
     &                         Uwind, Vwind,                            &
# else
     &                         sustr, svstr,                            &
# endif
#endif
#ifdef CARBON
     &                         pH,                                      &
#endif
#ifdef DIAGNOSTICS_BIO
     &                         DiaBio2d, DiaBio3d,                      &
#endif
!***************************8****************************************************
     &                         t)
!-----------------------------------------------------------------------
!
      USE mod_param
      USE mod_biology
      USE mod_ncparam
      USE mod_scalars
      USE ecodynamocpp_mod
#ifdef ICE_MODEL
     USE mod_ice
#endif

!
!  Imported variable declarations.
!
      integer, intent(in) :: ng, tile
      integer, intent(in) :: LBi, UBi, LBj, UBj, UBk, UBt
      integer, intent(in) :: IminS, ImaxS, JminS, JmaxS
      integer, intent(in) :: nstp, nnew
!****************************EcoDynamo stuff*********************************************
#ifdef ICE_MODEL
      integer, intent(in) :: liold, linew ! Pedro 17/12/2014
#endif
!****************************************************************************************

#ifdef ASSUMED_SHAPE
# ifdef MASKING
      real(r8), intent(in) :: rmask(LBi:,LBj:)
# endif
      real(r8), intent(in) :: Hz(LBi:,LBj:,:)
      real(r8), intent(in) :: z_r(LBi:,LBj:,:)
      real(r8), intent(in) :: z_w(LBi:,LBj:,0:)
      real(r8), intent(in) :: srflx(LBi:,LBj:)
!***********************Stuff imported from fennel.h*****************************8*******
# ifdef CARBON
      real(r8), intent(inout) :: pH(LBi:,LBj:)
# endif
# if defined CARBON || defined OXYGEN
#  ifdef BULK_FLUXES
      real(r8), intent(in) :: Uwind(LBi:,LBj:)
      real(r8), intent(in) :: Vwind(LBi:,LBj:)
#  else
      real(r8), intent(in) :: sustr(LBi:,LBj:)
      real(r8), intent(in) :: svstr(LBi:,LBj:)
#  endif
# endif
# ifdef DIAGNOSTICS_BIO
      real(r8), intent(inout) :: DiaBio2d(LBi:,LBj:,:)
      real(r8), intent(inout) :: DiaBio3d(LBi:,LBj:,:,:)
# endif
!****************************************************************************************
      real(r8), intent(inout) :: t(LBi:,LBj:,:,:,:)
!****************************EcoDynamo stuff*********************************************
#ifdef ICE_MODEL
      real(r8), intent(in) :: ai(LBi:,LBj:,:) !Pedro 17/12/14
      real(r8), intent(in) :: hi(LBi:,LBj:,:) !Pedro 22/06/15
      real(r8), intent(in) :: hsn(LBi:,LBj:,:) !Pedro 22/06/15
#endif
# ifdef CICE_COUPLING
      real(r8), intent(in) :: aice(LBi:,LBj:)
#endif
!****************************************************************************************      

#else
# ifdef MASKING
      real(r8), intent(in) :: rmask(LBi:UBi,LBj:UBj)
# endif
      real(r8), intent(in) :: Hz(LBi:UBi,LBj:UBj,UBk)
      real(r8), intent(in) :: z_r(LBi:UBi,LBj:UBj,UBk)
      real(r8), intent(in) :: z_w(LBi:UBi,LBj:UBj,0:UBk)
      real(r8), intent(in) :: srflx(LBi:UBi,LBj:UBj)
!****************************Stuff imported from fennel.h********************************
# ifdef CARBON
      real(r8), intent(inout) :: pH(LBi:UBi,LBj:UBj) 
# endif
# if defined CARBON || defined OXYGEN
#  ifdef BULK_FLUXES
      real(r8), intent(in) :: Uwind(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: Vwind(LBi:UBi,LBj:UBj)
#  else
      real(r8), intent(in) :: sustr(LBi:UBi,LBj:UBj)
      real(r8), intent(in) :: svstr(LBi:UBi,LBj:UBj)
#  endif
# endif
# ifdef DIAGNOSTICS_BIO
      real(r8), intent(inout) :: DiaBio2d(LBi:UBi,LBj:UBj,NDbio2d)
      real(r8), intent(inout) :: DiaBio3d(LBi:UBi,LBj:UBj,UBk,NDbio3d)
# endif
!****************************************************************************************
      real(r8), intent(inout) :: t(LBi:UBi,LBj:UBj,UBk,3,UBt)
!****************************EcoDynamo stuff*********************************************
#ifdef ICE_MODEL
      real(r8), intent(in) :: ai(LBi:UBi,LBj:UBj,2) !Pedro 17/12/14
      real(r8), intent(in) :: hi(LBi:,LBj:,:) !Pedro 22/06/15
      real(r8), intent(in) :: hsn(LBi:,LBj:,:) !Pedro 22/06/15
#endif
#ifdef CICE_COUPLING
      real(r8), intent(in) :: aice(LBi:UBi,LBj:UBj)
#endif
!****************************************************************************************
#endif
!
!  Local variable declarations.
!
#ifdef ECODYNAMO
!****************************Stuff imported from fennel.h********************************
# ifdef OXYGEN
      real(r8) :: u10squ
# endif
# ifdef CARBON
      real(r8), dimension(IminS:ImaxS) :: pCO2
# endif
!****************************************************************************************
      integer, parameter :: Nsink = 7
      integer :: iday, month, year
      integer :: casep
      real(r8) :: yday
      real(r8) :: hour
      real(r8), dimension(IminS:ImaxS,N(ng)) :: GrossProduction
      real(r8), dimension(IminS:ImaxS) :: PARUnderIce !Pedro 17/12/14
      real(r8), dimension(IminS:ImaxS) :: PAROnTheIce !Pedro 17/12/14
      real(r8) :: cffCResp 
      real(r8) :: cffCExud 
      real(r8) :: dumNitrite, cffNH4, cffNO3NO2, cffPO4, cffO2, cffTIC
!      real(r8) :: cffNH4Denit, cffNO3NO2Denit
      real(r8), dimension(1:Phyto_groups):: RhoChl
      real(r8) :: Chl2Carbon
      real(r8) :: NCellQuota;
      real(r8) :: PCellQuota; 
      real(r8), dimension(IminS:ImaxS,N(ng)) :: OxygenProduction
      real(r8), dimension(IminS:ImaxS,N(ng)) :: OxygenConsumption
      real(r8), dimension(IminS:ImaxS,N(ng)) :: CarbonProduction
      real(r8), dimension(IminS:ImaxS,N(ng)) :: CarbonConsumption
      real(r8), dimension(LBi:UBi,LBj:UBj) :: VwindBack
#else
      integer, parameter :: Nsink = 2
#endif
      integer :: Iter, i, ibio, isink, itime, itrc, iTrcMax, j, k, ks, ivar

      integer, dimension(Nsink) :: idsink

      real(r8), parameter :: MinVal = 1.0e-6_r8

      real(r8) :: Att, ExpAtt, Itop, PAR
      real(r8) :: cff, cff1, cff2, cff3, cff4, dtdays
      real(r8) :: cffL, cffR, cu, dltL, dltR
     
      real(r8), dimension(Nsink) :: Wbio

      integer, dimension(IminS:ImaxS,N(ng)) :: ksource

      real(r8), dimension(IminS:ImaxS) :: PARsur 


      real(r8), dimension(NT(ng),2) :: BioTrc

      real(r8), dimension(IminS:ImaxS,N(ng),NT(ng)) :: Bio
      real(r8), dimension(IminS:ImaxS,N(ng),NT(ng)) :: Bio_bak

      real(r8), dimension(IminS:ImaxS,0:N(ng)) :: FC

      real(r8), dimension(IminS:ImaxS,N(ng)) :: Hz_inv
      real(r8), dimension(IminS:ImaxS,N(ng)) :: Hz_inv2
      real(r8), dimension(IminS:ImaxS,N(ng)) :: Hz_inv3
      real(r8), dimension(IminS:ImaxS,N(ng)) :: Light
#ifdef ECODYNAMO
!****************************EcoDynamo stuff*********************************************
      real(r8), dimension(IminS:ImaxS,N(ng)) :: TopLight
      real(r8), dimension(IminS:ImaxS,N(ng)) :: BottomLight
      real(r8), dimension(IminS:ImaxS,N(ng)) :: KValue
      real(r8), dimension(IminS:ImaxS,N(ng)) :: LayerThickness
!****************************************************************************************
#endif
      real(r8), dimension(IminS:ImaxS,N(ng)) :: WL
      real(r8), dimension(IminS:ImaxS,N(ng)) :: WR
      real(r8), dimension(IminS:ImaxS,N(ng)) :: bL
      real(r8), dimension(IminS:ImaxS,N(ng)) :: bR
      real(r8), dimension(IminS:ImaxS,N(ng)) :: qc

#include "set_bounds.h"
!
#ifdef DIAGNOSTICS_BIO
!
!-----------------------------------------------------------------------
! If appropriate, initialize time-averaged diagnostic arrays.
!-----------------------------------------------------------------------
!
!     WRITE(6,*) 'NDbio2d = ',NDbio2d
!      IF (((iic(ng).gt.ntsDIA(ng)).and.                                 &
!     &     (MOD(iic(ng),nDIA(ng)).eq.1)).or.                            &
!     &    ((nrrec(ng).gt.0).and.(iic(ng).eq.ntstart(ng)))) THEN
        DO ivar=1,NDbio2d
!         WRITE(6,*) 'ivar =',ivar
          DO j=Jstr,Jend
            DO i=Istr,Iend
              DiaBio2d(i,j,ivar)=0.0_r8
            END DO
          END DO
        END DO
        DO ivar=1,NDbio3d
!          WRITE(6,*) 'NDbio3d =',NDbio3d
!          WRITE(6,*) 'ivar =',ivar
          DO k=1,N(ng)
            DO j=Jstr,Jend
              DO i=Istr,Iend
                DiaBio3d(i,j,k,ivar)=0.0_r8
              END DO
            END DO
          END DO
        END DO
!      END IF
#endif
# if defined CARBON || defined OXYGEN && defined BULK_FLUXES
       DO j=LBj,UBj
          DO i=LBi,UBi
                VwindBack(i,j)=Vwind(i,j);
           END DO
       END DO
#endif

    
!-----------------------------------------------------------------------
!  Add biological Source/Sink terms.
!-----------------------------------------------------------------------
!
!  Set time-stepping size (days) according to the number of iterations.
!
      dtdays=dt(ng)*sec2day/REAL(BioIter(ng),r8)
!
!  Set vertical sinking indentification vector.
!
      idsink(1)=iphyt                 ! Phytoplankton
      idsink(2)=iSdet                 ! Small detritus
#ifdef ECODYNAMO
!****************************EcoDynamo stuff**************************************************************
      idsink(3)=iPhyN                ! Phytoplankton nitrogen
      idsink(4)=iPhyP                ! Phytoplankton phosphorus
      idsink(5)=iSDeN                ! Small detritus nitrogen 
      idsink(6)=iSDeP                ! Small detritus phosphorus 
      idsink(7)=iChlo                ! Chlorophyll  
!*********************************************************************************************************
#endif
!
!  Set vertical sinking velocity vector in the same order as the
!  identification vector, IDSINK.
!
      Wbio(1)=wPhy(ng)                 ! Phytoplankton
      Wbio(2)=wDet(ng)                 ! Small detritus
#ifdef ECODYNAMO
!****************************EcoDynamo stuff**************************************************************
      Wbio(3)=wPhy(ng)                ! Phytoplankton nitrogen
      Wbio(4)=wPhy(ng)                ! Phytoplankton phosphorus
      Wbio(5)=wDet(ng)                ! Small detritus nitrogen
      Wbio(6)=wDet(ng)                ! Small detritus phosphorus
      Wbio(7)=wPhy(ng)                ! Chlorophyll  
!***********************************EcoDynamo stuff*******************************************************
   CALL caldate (r_date, tdays(ng), year, yday, month, iday, hour)
!*********************************************************************************************************
   !By default, it is assumed that the ratio of chlorophyll synthesis to carbon fixation = maximal chlorophyll : carbon ratio
   !If light is available RhoChl will be reduced by the equation shown below
      !RhoChl(1) = Chl2C_m(1,ng)
      !RhoChl(2) = Chl2C_m(2,ng)
      RhoChl(1) = Chl2C_m(ng)
      RhoChl(2) = Chl2C_m(ng)
#endif

!  Compute inverse thickness to avoid repeated divisions.
!
     cff = 0.0_r8
     cff1 = 0.0_r8

      cffCResp = 0.0_r8
      cffCExud = 0.0_r8
      J_LOOP : DO j=Jstr,Jend
!        IF (tile == 1) THEN
!           WRITE(6,*) 'jloop begin',j,Jstr,Jend
!        END IF
        DO k=1,N(ng)
          DO i=Istr,Iend
            Hz_inv(i,k)=1.0_r8/Hz(i,j,k)
          END DO
        END DO
        DO k=1,N(ng)-1
          DO i=Istr,Iend
            Hz_inv2(i,k)=1.0_r8/(Hz(i,j,k)+Hz(i,j,k+1))
          END DO
        END DO
        DO k=2,N(ng)-1
          DO i=Istr,Iend
            Hz_inv3(i,k)=1.0_r8/(Hz(i,j,k-1)+Hz(i,j,k)+Hz(i,j,k+1))
          END DO
        END DO
!
!  Restrict biological tracer to be positive definite. If a negative
!  concentration is detected, nitrogen is drawn from the most abundant
!  pool to supplement the negative pools to a lower limit of MinVal
!  which is set to 1E-6 above.
!
        DO k=1,N(ng)
          DO i=Istr,Iend
!
!  At input, all tracers (index nnew) from predictor step have
!  transport units (m Tunits) since we do not have yet the new
!  values for zeta and Hz. These are known after the 2D barotropic
!  time-stepping.
!
            DO itrc=1,NBT
              ibio=idbio(itrc)
              !IF ((itrc.gt.15).AND.(tile.eq.1).AND.(i.eq.Iend).AND.(k.eq.10)) THEN
              !   WRITE(*,*) 
              !   WRITE(*,*) 'itrc=',itrc,' idbio=',idbio(itrc)
              !   WRITE(*,*) 't(i,j,k,nstp,ibio)=',t(i,j,k,nstp,ibio)
              !   WRITE(*,*) 't(i,j,k,nnew,ibio)=',t(i,j,k,nnew,ibio)
              !END IF
              BioTrc(ibio,nstp)=t(i,j,k,nstp,ibio)
              BioTrc(ibio,nnew)=t(i,j,k,nnew,ibio)*Hz_inv(i,k)
            END DO
!
!  Impose positive definite concentrations.
!
            cff2=0.0_r8
            DO itime=1,2
              cff1=0.0_r8
              iTrcMax=idbio(1)
              DO itrc=1,NBT
                ibio=idbio(itrc)
                cff1=cff1+MAX(0.0_r8,MinVal-BioTrc(ibio,itime))
                IF (BioTrc(ibio,itime).gt.BioTrc(iTrcMax,itime)) THEN
                  iTrcMax=ibio
                END IF
                BioTrc(ibio,itime)=MAX(MinVal,BioTrc(ibio,itime))
              END DO
              IF (BioTrc(iTrcMax,itime).gt.cff1) THEN
                BioTrc(iTrcMax,itime)=BioTrc(iTrcMax,itime)-cff1
              END IF
            END DO
!
!  Load biological tracers into local arrays.
!
            DO itrc=1,NBT
              ibio=idbio(itrc)
              Bio_bak(i,k,ibio)=BioTrc(ibio,nstp)
              Bio(i,k,ibio)=BioTrc(ibio,nstp)
            END DO
          END DO
        END DO
!
!  Calculate surface Photosynthetically Available Radiation (PAR).  The
!  net shortwave radiation is scaled back to Watts/m2 and multiplied by
!  the fraction that is photosynthetically available, PARfrac.
!
        DO i=Istr,Iend
#ifdef CONST_PAR
!
!  Specify constant surface irradiance a la Powell and Spitz.
!
          PARsur(i)=158.075_r8
          PARUnderIce(i)=0_r8
#else
!          WRITE(0,*) 'ecodynamo'
          PARsur(i)=PARfrac(ng)*srflx(i,j)*rho0*Cp

#endif
        END DO
!
!=======================================================================
!  Start internal iterations to achieve convergence of the nonlinear
!  backward-implicit solution.
!=======================================================================
!
!  During the iterative procedure a series of fractional time steps are
!  performed in a chained mode (splitting by different biological
!  conversion processes) in sequence of the main food chain.  In all
!  stages the concentration of the component being consumed is treated
!  in a fully implicit manner, so the algorithm guarantees non-negative
!  values, no matter how strong the concentration of active consuming
!  component (Phytoplankton or Zooplankton).  The overall algorithm,
!  as well as any stage of it, is formulated in conservative form
!  (except explicit sinking) in sense that the sum of concentration of
!  all components is conserved.
!
!  In the implicit algorithm, we have for example (N: nutrient,
!                                                  P: phytoplankton),
!
!     N(new) = N(old) - uptake * P(old)     uptake = mu * N / (Kn + N)
!                                                    {Michaelis-Menten}
!  below, we set
!                                           The N in the numerator of
!     cff = mu * P(old) / (Kn + N(old))     uptake is treated implicitly
!                                           as N(new)
!
!  so the time-stepping of the equations becomes:
!
!     N(new) = N(old) / (1 + cff)     (1) when substracting a sink term,
!                                         consuming, divide by (1 + cff)
!  and
!
!     P(new) = P(old) + cff * N(new)  (2) when adding a source term,
!                                         growing, add (cff * source)
!
!  Notice that if you substitute (1) in (2), you will get:
!
!     P(new) = P(old) + cff * N(old) / (1 + cff)    (3)
!
!  If you add (1) and (3), you get
!
!     N(new) + P(new) = N(old) + P(old)
!
!  implying conservation regardless how "cff" is computed. Therefore,
!  this scheme is unconditionally stable regardless of the conversion
!  rate. It does not generate negative values since the constituent
!  to be consumed is always treated implicitly. It is also biased
!  toward damping oscillations.
!
!  The iterative loop below is to iterate toward an universal Backward-
!  Euler treatment of all terms. So if there are oscillations in the
!  system, they are only physical oscillations. These iterations,
!  however, do not improve the accuaracy of the solution.
!
!        WRITE(6,*) 'bioiter', BioIter(ng) 
        ITER_LOOP: DO Iter=1,BioIter(ng)
!
!  Compute light attenuation as function of depth.
!

!        WRITE(6,*) 'Iter', Iter
          DO i=Istr,Iend
            PAR=PARsur(i)      
!            WRITE(6,*) 'tile,i,j', i,j,N(ng)                   
            DO k=N(ng),1,-1   
!               WRITE(6,*) 'k', k               
!  Compute layer thickness and average light attenuation for each grid cell. Here, AttSW is
!  the light attenuation due to seawater and AttPhy is the attenuation
!  due to phytoplankton (self-shading coefficient).
                LayerThickness(i,k) = (z_w(i,j,k)-z_w(i,j,k-1))   
                IF (PARsur(i).gt.0.0_r8) THEN         ! day time
#ifdef K160_MODEL
                   KValue(i,k)=(AttSW(ng)+AttPhy(ng)*Bio(i,k,iPhyt)+               &
                    MAX(-0.1409_r8*t(i,j,k,nstp,isalt)+5.1695_r8,0.0_r8))
#else
                   KValue(i,k)=(AttSW(ng)+AttPhy(ng)*Bio(i,k,iPhyt))
#endif
!                  KValue(i,k)=(AttSW(ng)+AttPhy(ng)*Bio(i,k,iPhyt))
!                  LayerThickness(i,k) = (z_w(i,j,k)-z_w(i,j,k-1))
                   Att=KValue(i,k)* LayerThickness(i,k)
                   ExpAtt=EXP(-Att)
                   Itop=PAR
                   TopLight(i,k) = PAR
                   PAR=Itop*(1.0_r8-ExpAtt)/Att    ! average at cell center
                   Light(i,k)=PAR
!
!  Light attenuation at the bottom of the grid cell. It is the starting
!  PAR value for the next (deeper) vertical grid cell.
!
                   PAR=Itop*ExpAtt
                   BottomLight(i,k)=PAR 
                ELSE                                 ! night time
                   Light(i,k)=0.0_r8
                   TopLight(i,k)=0.0_r8  
                   BottomLight(i,k)=0.0_r8 
                END IF 
              END DO
          END DO
!
!  Phytoplankton photosynthetic growth and nitrate uptake (Vm_NO3 rate).
!  The Michaelis-Menten curve is used to describe the change in uptake
!  rate as a function of nitrate concentration. Here, PhyIS is the
!  initial slope of the P-I curve and K_NO3 is the half saturation of
!  phytoplankton nitrate uptake.
!
#ifndef ECODYNAMO
# ifdef SPITZ
          cff1=dtdays*Vm_NO3(ng)*PhyIS(ng)
          cff2=Vm_NO3(ng)*Vm_NO3(ng)
          cff3=PhyIS(ng)*PhyIS(ng)
 #else
          cff1=dtdays*Vm_NO3(ng)
# endif
#endif
          DO k=1,N(ng)
            DO i=Istr,Iend
#ifdef ECODYNAMO
!***********************************EcoDynamo calculations*******************************************************
!              CALL phytoplankton_go( PHYOBJ1,Hz(i,j,k),dt(ng))
               CALL phytoplankton_go( PHYOBJ1,LayerThickness(i,k),dt(ng))
               CALL phytoplankton_go( PHYOBJ2,LayerThickness(i,k),dt(ng))
               CALL dissobjt_go(DISSOBJ1,LayerThickness(i,k),dt(ng))
               GrossProduction(i,k) = 0.0_r8
               OxygenProduction(i,k) = 0.0_r8
               OxygenConsumption(i,k) = 0.0_r8
               CarbonProduction(i,k) = 0.0_r8
               CarbonConsumption(i,k) = 0.0_r8 

               IF (TopLight(i,k).gt.MinVal) THEN 
                  casep=4
                  if (Bio(i,k,iPhyt).gt.MinVal) then
                     Chl2Carbon = Bio(i,k,iChlo) /                      &
      &                           (Bio(i,k,iPhyt) * 12.0_r8)    
                  else
                     Chl2Carbon = MinVal
                  end if
                   
                  CALL phytoplankton_production(PHYOBJ1,                &
     &                                            TopLight(i,k),        & 
     &                                            BottomLight(i,k),     &
     &                                            KValue(i,k),          &
     &                                            t(i,j,k,nstp,itemp),  &
     &                                            casep, yday,          &
     &                                            GrossProduction(i,k), &
     &                                            Bio(i,k,iPhyN),       &
     &                                            Bio(i,k,iPhyP),       &
     &                                            Bio(i,k,iPhyt),       &
     &                                            Bio(i,k,iTIC_),       &
     &                                            PhyIS(ng),            &
     &                                            Chl2Carbon,           &
     &                                            OxygenProduction(i,k))

                 
                  cff = (PhyIS(ng) * Light(i,k) * Bio(i,k,iPhyt))

!***Calculation of the fraction of phytoplankton growth devoted to chlorophyll synthesis (see Fennel model)*************************************************************************
!Using a formula comparable to that employed by Fennel, exccept for the differing units of the initial slope. The numerator contains achievable production 
!(GrossProduction [mmol C m-3 s-1]) and the denominator represents potential production (cff):

                  if (cff.gt.MinVal) then 
                      !RhoChl(1) = Chl2C_m(1,ng) * GrossProduction(i,k) /     & 
                      RhoChl(1) = Chl2C_m(ng) * GrossProduction(i,k) /        &    
    &                          cff  
                  endif 
                  
                  Bio(i,k,iChlo)=Bio(i,k,iChlo)+GrossProduction(i,k) *  &
    &                             12.0_r8 * RhoChl(1) *                    & 
    &                              dt(ng)/REAL(BioIter(ng),r8) 
!Chlorophyll units mg Chl / m3   
!*********************************************************************************************************************************************************************************** 
! Here we are not applying the backward-implicit solution because primary production is not calculated as 
! a function of TIC at the end of the time-step (tipically TIC is very abundant in sea water....)
! However, to prevent the possibility of having production without TIC in the water,
! when CARBON is defined production occurs only if TIC is larger than a given threshold (see above)

                  cff1 = GrossProduction(i,k)*dt(ng)/REAL(BioIter(ng),r8) 
                  cff2 = OxygenProduction(i,k)*dt(ng)/REAL(BioIter(ng),r8)
# if defined CARBON               
                  Bio(i,k,iTIC_) = Bio(i,k,iTIC_) - cff1  
                  CarbonConsumption(i,k) = CarbonConsumption(i,k)+cff1    
# endif   
                  Bio(i,k,iPhyt)=Bio(i,k,iPhyt)+ cff1   !Carbon units mmol C / m3 
# if defined OXYGEN 
                  Bio(i,k,iOxyg)=Bio(i,k,iOxyg) + cff2  !Oxygen units mmol O2 / m3 
# endif
              ENDIF
  
#ifdef DIAGNOSTICS_BIO
                DiaBio3d(i,j,k,iPPro)= DiaBio3d(i,j,k,iPPro)+           &
# ifdef WET_DRY
     &                                rmask_io(i,j)*                    &
# endif
     &                                GrossProduction(i,k)*             &
     &                                dt(ng)/REAL(BioIter(ng),r8)
#endif   
               cffO2 = 0.0_r8
               cffNH4 = 0.0_r8 
               cffNO3NO2 = 0.0_r8
               dumNitrite = 0.0_r8
!               cffNH4Denit = 0.0_r8
!               cffNO3NO2Denit = 0.0_r8
               ! Pedro - 16/11/2016
               CALL dissobjt_nitrification(DISSOBJ1,                    &
     &                                            TopLight(i,k),        & 
     &                                            BottomLight(i,k),     &
     &                                            KValue(i,k),          &
     &                                            t(i,j,k,nstp,itemp),  &
     &                                            Bio(i,k,iNH4_),       & 
     &                                            Bio(i,k,iOxyg),       &
     &                                            cffNO3NO2,            &
     &                                            cffO2)
! Pedro - 16/11/2016
! The division of cffO2 by Bio(i,k,iNH4_) and its later multiplication by
! the updated value of the same variable is to correct the oxygen flux
! and make it compatible with the other backward-implict fluxes

                !cffO2 = cffO2 / Bio(i,k,iNH4_)  
                if (Bio(i,k,iNH4_).gt.MinVal) then
                   cffNH4 = cffNO3NO2*dt(ng)/                   &
     &                      REAL(BioIter(ng),r8)/Bio(i,k,iNH4_)
                else
                   cffNH4 = 0.0_r8 
                endif 
                Bio(i,k,iNH4_)=Bio(i,k,iNH4_)/(1.0_r8+cffNH4)   
                Bio(i,k,iNO3_)=Bio(i,k,iNO3_) + cffNH4 * Bio(i,k,iNH4_) 
                

# if defined OXYGEN 
    !            cffO2 = cffO2 * Bio(i,k,iNH4_) 
    !            Bio(i,k,iOxyg) = Bio(i,k,iOxyg)-                        &
    ! &                           cffO2*dt(ng)/REAL(BioIter(ng),r8)
                OxygenConsumption(i,k) = OxygenConsumption(i,k) +       & 
     &                           cffO2*dt(ng)/REAL(BioIter(ng),r8) 
                if (Bio(i,k,iOxyg).gt.MinVal) then
                   cffO2 = cffO2*dt(ng)/                   &
     &                      REAL(BioIter(ng),r8)/Bio(i,k,iOxyg)
                else
                   cffO2 = 0.0_r8 
                endif
                Bio(i,k,iOxyg)=Bio(i,k,iOxyg)/(1.0_r8+cffO2)
                
# endif               
                               
! Pedro - 16/11/2016                
               CALL dissobjt_denitrification(DISSOBJ1,                  &
     &                                            t(i,j,k,nstp,itemp),  &
     &                                            Bio(i,k,iNO3_),       & 
     &                                            Bio(i,k,iOxyg),       &
     &                                            cffNH4,               &
     &                                            cffNO3NO2)

!cffNH4 <= cffNO3NO2 because denitritication may not result only in NH4+
!Therefore, unlike in nitrification we must keep track of these two
!possibly different fluxes (ammonia gain and nitrate losses)

               if (Bio(i,k,iNH4_).gt.MinVal) then  
                  cffNH4 = cffNH4*dt(ng)/REAL(BioIter(ng),r8)/          &
     &                     Bio(i,k,iNO3_)
               else
                  cffNH4 = 0.0_r8  
               endif 
! Pedro - 16/11/2016
               if (Bio(i,k,iNO3_).gt.MinVal) then
                  cffNO3NO2 = cffNO3NO2*dt(ng)/                         &
     &                          REAL(BioIter(ng),r8)/Bio(i,k,iNO3_)
               else
                  cffNO3NO2 = 0.0_r8 
               endif    
               Bio(i,k,iNO3_)=Bio(i,k,iNO3_)/(1.0_r8+cffNO3NO2)
               Bio(i,k,iNH4_)=Bio(i,k,iNH4_) + cffNH4 * Bio(i,k,iNO3_)    
! Pedro - 16/11/2016
               

               CALL phytoplankton_nitrogen_uptake( PHYOBJ1,             &
     &                                             Bio(i,k,iNH4_),      & 
     &                                             Bio(i,k,iNO3_),      &
     &                                             dumNitrite, cffNH4,  &
     &                                             cffNO3NO2,           &
     &                                             Bio(i,k,iPhyN),      &
     &                                             Bio(i,k,iPhyt))
 
               if (Bio(i,k,iNH4_).gt.MinVal) then
                  cffNH4 = cffNH4*dt(ng)/REAL(BioIter(ng),r8)/          &
      &                    Bio(i,k,iNH4_)
               else
                  cffNH4 = 0.0_r8  
               endif 
               Bio(i,k,iNH4_)=Bio(i,k,iNH4_)/(1.0_r8+cffNH4)  
               if (Bio(i,k,iNO3_).gt.MinVal) then 
                  cffNO3NO2 = cffNO3NO2*dt(ng)/REAL(BioIter(ng),r8)/    & 
     &                     Bio(i,k,iNO3_)
               else
                  cffNO3NO2 = 0.0_r8  
               endif 
               Bio(i,k,iNO3_)=Bio(i,k,iNO3_)/(1.0_r8+cffNO3NO2) 
! Pedro - 16/11/2016      
               Bio(i,k,iPhyN)=Bio(i,k,iPhyN)+ Bio(i,k,iNH4_)* cffNH4    &
     &                        +Bio(i,k,iNO3_)*cffNO3NO2
               

               cffPO4 = 0.0_r8
               CALL phytoplankton_phosphorus_uptake(PHYOBJ1,            &
     &                                               Bio(i,k,iPO4_),    &
     &                                               cffPO4,            &
     &                                               Bio(i,k,iPhyP),    &
     &                                               Bio(i,k,iPhyt))  
               if (Bio(i,k,iPO4_).gt.MinVal) then 
                  cffPO4=cffPO4*dt(ng)/REAL(BioIter(ng),r8)/            &
     &            Bio(i,k,iPO4_)
               else
                  cffPO4= 0.0_r8
               endif
              
               Bio(i,k,iPO4_)=Bio(i,k,iPO4_)/(1.0_r8+cffPO4)    
               Bio(i,k,iPhyP)=Bio(i,k,iPhyP)+ Bio(i,k,iPO4_)*cffPO4 
!****************************************************************************************************************
#else
# ifdef SPITZ
              cff4=1.0_r8/SQRT(cff2+cff3*Light(i,k)*Light(i,k))
              cff=Bio(i,k,iPhyt)*                                       &
     &            cff1*cff4*Light(i,k)/                                 &
     &            (K_NO3(ng)+Bio(i,k,iNO3_))
# else
              cff=Bio(i,k,iPhyt)*                                       &
     &            cff1*Light(i,k)/                                      &
     &            (K_NO3(ng)+Bio(i,k,iNO3_))
# endif
              Bio(i,k,iNO3_)=Bio(i,k,iNO3_)/(1.0_r8+cff)
              Bio(i,k,iPhyt)=Bio(i,k,iPhyt)+                            &
     &                       Bio(i,k,iNO3_)*cff
#endif  
            END DO
          END DO
!
!  Grazing on phytoplankton by zooplankton (ZooGR rate) using the Ivlev
!  formulation (Ivlev, 1955) and lost of phytoplankton to the nitrate
!  pool as function of "sloppy feeding" and metabolic processes
!  (ZooEEN and ZooEED fractions).
!

          cff1=dtdays*ZooGR(ng)
          cff2=1.0_r8-ZooEEN(ng)-ZooEED(ng)
          DO k=1,N(ng)
            DO i=Istr,Iend
              IF (ABS(Bio(i,k,iPhyt)) .lt. 1.0E-10) THEN
              cff = 0.0_r8
              ELSE
              cff=Bio(i,k,iZoop)*                                       &
     &            cff1*(1.0_r8-EXP(-Ivlev(ng)*Bio(i,k,iPhyt)))/         &
     &            Bio(i,k,iPhyt)
              END IF
              Bio(i,k,iPhyt)=Bio(i,k,iPhyt)/(1.0_r8+cff)
              Bio(i,k,iZoop)=Bio(i,k,iZoop)+                            &
     &                       Bio(i,k,iPhyt)*cff2*cff
#ifdef ECODYNAMO
!***********************************EcoDynamo calculations*******************************************************
              Bio(i,k,iPhyN)=Bio(i,k,iPhyN)/(1.0_r8+cff)
              Bio(i,k,iZopN)=Bio(i,k,iZopN)+                            &
     &                       Bio(i,k,iPhyN)*cff2*cff
              Bio(i,k,iPhyP)=Bio(i,k,iPhyP)/(1.0_r8+cff)
              Bio(i,k,iZopP)=Bio(i,k,iZopP)+                            &
     &                       Bio(i,k,iPhyP)*cff2*cff
              Bio(i,k,iNH4_)=Bio(i,k,iNH4_)+                            &
     &                       Bio(i,k,iPhyN)*ZooEEN(ng)*cff
              Bio(i,k,iSDeN)=Bio(i,k,iSDeN)+                            &
     &                       Bio(i,k,iPhyN)*ZooEED(ng)*cff   
              Bio(i,k,iSDeP)=Bio(i,k,iSDeP)+                            &
     &                       Bio(i,k,iPhyP)*ZooEED(ng)*cff
!****************************************************************************************************************
#endif
#ifndef ECODYNAMO
              Bio(i,k,iNO3_)=Bio(i,k,iNO3_)+                            &
     &                       Bio(i,k,iPhyt)*ZooEEN(ng)*cff
#endif
              Bio(i,k,iSDet)=Bio(i,k,iSDet)+                            &
     &                       Bio(i,k,iPhyt)*ZooEED(ng)*cff
            END DO
          END DO

!  Phytoplankton mortality to nutrients (PhyMRN rate) and detritus
!  (PhyMRD rate).
!
          cff3=dtdays*PhyMRD(ng)
#ifndef ECODYNAMO
          cff2=dtdays*PhyMRN(ng)
          cff1=1.0_r8/(1.0_r8+cff2+cff3)
#endif
          DO k=1,N(ng)
            DO i=Istr,Iend
#ifdef ECODYNAMO  
!***********************************EcoDynamo calculations*******************************************************
              cffCResp = 0.0_r8 
              if (Bio(i,k,iPhyt).gt.MinVal) then
                 Chl2Carbon = Bio(i,k,iChlo) /                          &
       &                      (Bio(i,k,iPhyt) * 12.0_r8)  
                 NCellQuota = Bio(i,k,iPhyN) / Bio(i,k,iPhyt);
                 PCellQuota = Bio(i,k,iPhyP) / Bio(i,k,iPhyt);  
              else
                 Chl2Carbon = MinVal
              end if

              CALL phytoplankton_respiration (PHYOBJ1,                  &
       &                                      t(i,j,k,nstp,itemp),      &
       &                                      cffCResp,                 &
       &                                      GrossProduction(i,k),     &
       &                                      Bio(i,k,iPhyt),           &
       &                                      Bio(i,k,iOxyg),           &
       &                                      Chl2Carbon,               &
       &                                      cffO2)  
               

! Phytoplankton respiration and exudation to nutrients
!
! The division of cffO2 by Bio(i,k,iPhyt) and its later multiplication by
! the updated value of the same variable is to correct the oxygen flux
! and make it compatible with the other backward-implict fluxes

              !cffO2 = cffO2 / Bio(i,k,iPhyt)
              if (Bio(i,k,iPhyt).gt.MinVal) then
                 cff2 = cffCResp*dt(ng)/REAL(BioIter(ng),r8)/            &
         &              Bio(i,k,iPhyt)
              else
                 cff2 = 0.0_r8
              endif
              Bio(i,k,iPhyt) = Bio(i,k,iPhyt)/(1.0_r8+cff2)
              Bio(i,k,iChlo)=Bio(i,k,iChlo)/(1.0_r8+cff2)
# if defined CARBON
              Bio(i,k,iTIC_) = Bio(i,k,iTIC_) + cff2 * Bio(i,k,iPhyt)
              CarbonProduction(i,k) = CarbonProduction(i,k) +           &
         &                            cff2 * Bio(i,k,iPhyt)
# endif
# if defined OXYGEN 
 !             cffO2 = cffO2 * Bio(i,k,iPhyt)          
 !             Bio(i,k,iOxyg) = Bio(i,k,iOxyg) -                         &
 !       &                      cffO2*dt(ng)/REAL(BioIter(ng),r8)
             
              OxygenConsumption(i,k) = OxygenConsumption(i,k) +         &
         &                      cffO2*dt(ng)/REAL(BioIter(ng),r8) 
              if (Bio(i,k,iOxyg).gt.MinVal) then
                   cffO2 = cffO2*dt(ng)/                   &
         &                  REAL(BioIter(ng),r8)/Bio(i,k,iOxyg)
              else
                   cffO2 = 0.0_r8 
              endif
              
              Bio(i,k,iOxyg)=Bio(i,k,iOxyg)/(1.0_r8+cffO2)
              
# endif 
              

               
                          
              cffCExud = 0.0_r8      
              CALL phytoplankton_exudation   (PHYOBJ1, cffCExud,        &
       &                                      GrossProduction(i,k),     &
       &                                      Bio(i,k,iPhyt),           &
       &                                      NCellQuota, PCellQuota)  
              if (Bio(i,k,iPhyt).gt.MinVal) then
                 cffCExud = cffCExud*dt(ng)/REAL(BioIter(ng),r8)        &
       &                 /Bio(i,k,iPhyt)  
              endif 
            
#ifdef DIAGNOSTICS_BIO
              DiaBio3d(i,j,k,iPPro)= DiaBio3d(i,j,k,iPPro)-             &
# ifdef WET_DRY
      &                                rmask_io(i,j)*                   &
# endif
     &                                Bio(i,k,iPhyt) * cff2  

#endif
 
! Respiration is leading to N and P losses but exudation only to C losses
              cff1=1.0_r8/(1.0_r8+cff2+cff3) 
              Bio(i,k,iPhyN)=Bio(i,k,iPhyN)*cff1
              Bio(i,k,iPhyP)=Bio(i,k,iPhyP)*cff1 
              Bio(i,k,iNH4_)=Bio(i,k,iNH4_)+                            &
     &                       Bio(i,k,iPhyN)*cff2

              Bio(i,k,iPO4_)=Bio(i,k,iPO4_)+                            &
     &                       Bio(i,k,iPhyP)*cff2
! cff2 and cff1 recalculated to account for C losses from exudation
              cff2 = cffCExud
              cff1=1.0_r8/(1.0_r8+cff2+cff3)
!****************************************************************************************************************
#endif
              Bio(i,k,iPhyt)=Bio(i,k,iPhyt)*cff1
#ifndef ECODYNAMO              
              Bio(i,k,iNO3_)=Bio(i,k,iNO3_)+                            &
     &                       Bio(i,k,iPhyt)*cff2
#endif
              Bio(i,k,iSDet)=Bio(i,k,iSDet)+                            &
     &                       Bio(i,k,iPhyt)*cff3
#ifdef ECODYNAMO
!***********************************EcoDynamo calculations*******************************************************
              Bio(i,k,iChlo)=Bio(i,k,iChlo)* cff1!Chlorophyll units mg Chl / m3 / dt  
              Bio(i,k,iSDeN)=Bio(i,k,iSDeN)+                            &
     &                       Bio(i,k,iPhyN)*cff3
              Bio(i,k,iSDeP)=Bio(i,k,iSDeP)+                            &
     &                       Bio(i,k,iPhyP)*cff3
!****************************************************************************************************************
#endif
            END DO
          END DO


!
!  Zooplankton mortality to nutrients (ZooMRN rate) and Detritus
!  (ZooMRD rate).
! 
          cff3=dtdays*ZooMRD(ng)
#ifdef ECODYNAMO
          cff2 = 0
#else
          cff2=dtdays*ZooMRN(ng)
#endif
          cff1=1.0_r8/(1.0_r8+cff2+cff3)
          DO k=1,N(ng)
            DO i=Istr,Iend
              Bio(i,k,iZoop)=Bio(i,k,iZoop)*cff1
#ifndef ECODYNAMO
              Bio(i,k,iNO3_)=Bio(i,k,iNO3_)+                            &
     &                       Bio(i,k,iZoop)*cff2
#endif
              Bio(i,k,iSDet)=Bio(i,k,iSDet)+                            &
     &                       Bio(i,k,iZoop)*cff3
#ifdef ECODYNAMO
!***********************************EcoDynamo calculations*******************************************************
              Bio(i,k,iZopN)=Bio(i,k,iZopN)*cff1
              Bio(i,k,iSDeN)=Bio(i,k,iSDeN)+                            &
     &                       Bio(i,k,iZopN)*cff3
              Bio(i,k,iZopP)=Bio(i,k,iZopP)*cff1
              Bio(i,k,iSDeP)=Bio(i,k,iSDeP)+                            &
     &                       Bio(i,k,iZopP)*cff3
!****************************************************************************************************************
#endif
            END DO
          END DO

!
!  Detritus breakdown to nutrients: remineralization (DetRR rate).
!
#ifndef ECODYNAMO
          cff2=dtdays*DetRR(ng)
          cff1=1.0_r8/(1.0_r8+cff2)
#endif


          DO k=1,N(ng)
            DO i=Istr,Iend
#ifdef ECODYNAMO
              CALL dissobjt_CarbonMineralization(DISSOBJ1,             &
     &                                     t(i,j,k,nstp,itemp),        &
     &                                     Bio(i,k,iSDet),             &
     &                                     Bio(i,k,iOxyg),             &
     &                                     cff2,                       &
     &                                     minRateC(ng)) 
              if (Bio(i,k,iSDet).gt.MinVal) then 
                 cff2 = cff2*dt(ng)/REAL(BioIter(ng),r8)/Bio(i,k,iSDet)
              else
                 cff2 = 0.0_r8
              endif  
              cff1=1.0_r8/(1.0_r8+cff2)
                      
#endif
              Bio(i,k,iSDet)=Bio(i,k,iSDet)*cff1
# if defined CARBON
              Bio(i,k,iTIC_) = Bio(i,k,iTIC_) + cff2 * Bio(i,k,iSDet)
              CarbonProduction(i,k) = CarbonProduction(i,k) +          &
     &                                cff2 * Bio(i,k,iSDet)
# endif
#ifndef ECODYNAMO
              Bio(i,k,iNO3_)=Bio(i,k,iNO3_)+                           &
     &                       Bio(i,k,iSDet)*cff2
#endif
#ifdef ECODYNAMO
              CALL dissobjt_NitrogenMineralization(DISSOBJ1,           &
     &                                     t(i,j,k,nstp,itemp),        &
     &                                     Bio(i,k,iSDeN),             &
     &                                     Bio(i,k,iOxyg),             &
     &                                     cff2,                       &
     &                                     cffO2,                      &
     &                                     minRateN(ng)) 

! The division of cffO2 by Bio(i,k,iPhyt) and its later multiplication by
! the updated value of the same variable is to correct the oxygen flux
! and make it compatible with the other backward-implict fluxes

              !cffO2 = cffO2 / Bio(i,k,iSDeN)
              if (Bio(i,k,iSDeN).gt.MinVal) then
                 cff2 = cff2*dt(ng)/REAL(BioIter(ng),r8)/Bio(i,k,iSDeN)
              else 
                 cff2 = 0.0_r8
              endif 
              cff1=1.0_r8/(1.0_r8+cff2)       
              Bio(i,k,iSDeN)=Bio(i,k,iSDeN)*cff1
              Bio(i,k,iNH4_)=Bio(i,k,iNH4_)+ cff2*Bio(i,k,iSDeN) 
# if defined OXYGEN
              !cffO2 = cffO2 * Bio(i,k,iSDeN)
              

!              Bio(i,k,iOxyg) = Bio(i,k,iOxyg) -                        &
!      &                        cffO2*dt(ng)/REAL(BioIter(ng),r8)  
              OxygenConsumption(i,k) = OxygenConsumption(i,k) +         &
      &                               cffO2*dt(ng)/REAL(BioIter(ng),r8) 
              if (Bio(i,k,iOxyg).gt.MinVal) then
                   cffO2 = cffO2*dt(ng)/                   &
     &                      REAL(BioIter(ng),r8)/Bio(i,k,iOxyg)
              else
                   cffO2 = 0.0_r8 
              endif
              Bio(i,k,iOxyg)=Bio(i,k,iOxyg)/(1.0_r8+cffO2)    
              
# endif 
      
              
                
 
             
              CALL dissobjt_PhosphorusMineralization(DISSOBJ1,         &
     &                                     t(i,j,k,nstp,itemp),        &
     &                                     Bio(i,k,iSDeP),             &
     &                                     Bio(i,k,iOxyg),             &
     &                                     cff2,                       &
     &                                     minRateP(ng)) 

              

              if (Bio(i,k,iSDeP).gt.MinVal) then
                 cff2 = cff2*dt(ng)/REAL(BioIter(ng),r8)/Bio(i,k,iSDeP)
              else
                 cff2 = 0.0_r8
              endif
              cff1=1.0_r8/(1.0_r8+cff2)       
         

              Bio(i,k,iSDeP)=Bio(i,k,iSDeP)*cff1
              
              Bio(i,k,iPO4_)=Bio(i,k,iPO4_)+ cff2 * Bio(i,k,iSDeP)
#endif  
            END DO
          END DO

# if defined ECODYNAMO 
#  if defined OXYGEN

     CALL     O2_exchange (ng, tile,Istr,Iend,                          &
     &                 LBi, UBi, LBj, UBj, IminS, ImaxS,                &
#  ifdef MASKING
     &                 rmask(IminS:,j),                                 &
#  endif
# ifdef BULK_FLUXES
     &                 Uwind(IminS:,j),Vwind(IminS:,j),                 &
# else
     &                 sustr(IminS:,j),svstr(IminS:,j),                 &
     &                 svstrNext(IminS:,j+1)                            &
# endif
     &                 t(IminS:,j,N(ng),nstp,itemp),                    &
     &                 t(IminS:,j,N(ng),nstp,isalt),                    &
     &                 Bio(IminS:,N(ng),iOxyg),dtdays,                  &
#ifdef DIAGNOSTICS_BIO
     &                 DiaBio2d(IminS:,j,iO2fx),                        &
#endif
     &                 LayerThickness(IminS:,N(ng)), rho0,              &
#ifdef ICE_MODEL
     &                 ai(IminS:,j,linew),                              &
#endif
#ifdef CICE_COUPLING
     &                 aice(IminS:,j),                                 & 
#endif
     &                  j)
# endif
#  if defined CARBON
     CALL  CO2_exchange(ng, tile,Istr,Iend,                           &
     &                LBi, UBi, LBj, UBj, IminS, ImaxS,               &
#ifdef MASKING
     &                rmask(IminS:,j),                                &
#endif
#ifdef BULK_FLUXES
     &                Uwind(IminS:,j),VwindBack(IminS:,j),            &
# else
     &                sustr(IminS:,j),svstr(IminS:,j),                &
     &                svstrNext(IminS:,j+1),                          & 
#endif
     &                t(IminS:,j,N(ng),nstp,itemp),                   &
     &                t(IminS:,j,N(ng),nstp,isalt),                   &
     &                Bio(IminS:,N(ng),iTIC_),                        &
     &                Bio(IminS:,N(ng),iTAlk),                        &
     &                pH(IminS:,j),                                   &
!     &                t(IminS:,j,N(ng),nstp,ipCO2),                  &
     &                pCO2,                                           &
     &                dtdays,                                         &
#ifdef DIAGNOSTICS_BIO
     &                DiaBio2d(IminS:,j,iCOfx),                       &
     &                DiaBio2d(IminS:,j,ipCO2),                       &
#endif
     &                LayerThickness(IminS:,N(ng)),                   &
     &                rho0,                                           &
#ifdef ICE_MODEL
     &                ai(IminS:,j,linew),                             &
#endif
#ifdef CICE_COUPLING
     &                aice(IminS:,j),                                 & 
#endif
     &                j, yday, pCO2air(ng))
#  endif
#endif
!
!-----------------------------------------------------------------------
!  Vertical sinking terms: Phytoplankton and Detritus
!-----------------------------------------------------------------------
!
!  Reconstruct vertical profile of selected biological constituents
!  "Bio(:,:,isink)" in terms of a set of parabolic segments within each
!  grid box. Then, compute semi-Lagrangian flux due to sinking.
!
          SINK_LOOP: DO isink=1,Nsink
            ibio=idsink(isink)
!
!  Copy concentration of biological particulates into scratch array
!  "qc" (q-central, restrict it to be positive) which is hereafter
!  interpreted as a set of grid-box averaged values for biogeochemical
!  constituent concentration.
!
            DO k=1,N(ng)
              DO i=Istr,Iend
                qc(i,k)=Bio(i,k,ibio)
              END DO
            END DO
!
            DO k=N(ng)-1,1,-1
              DO i=Istr,Iend
                FC(i,k)=(qc(i,k+1)-qc(i,k))*Hz_inv2(i,k)
              END DO
            END DO
            DO k=2,N(ng)-1
              DO i=Istr,Iend
                dltR=Hz(i,j,k)*FC(i,k)
                dltL=Hz(i,j,k)*FC(i,k-1)
                cff=Hz(i,j,k-1)+2.0_r8*Hz(i,j,k)+Hz(i,j,k+1)
                cffR=cff*FC(i,k)
                cffL=cff*FC(i,k-1)
!
!  Apply PPM monotonicity constraint to prevent oscillations within the
!  grid box.
!
                IF ((dltR*dltL).le.0.0_r8) THEN
                  dltR=0.0_r8
                  dltL=0.0_r8
                ELSE IF (ABS(dltR).gt.ABS(cffL)) THEN
                  dltR=cffL
                ELSE IF (ABS(dltL).gt.ABS(cffR)) THEN
                  dltL=cffR
                END IF
!
!  Compute right and left side values (bR,bL) of parabolic segments
!  within grid box Hz(k); (WR,WL) are measures of quadratic variations.
!
!  NOTE: Although each parabolic segment is monotonic within its grid
!        box, monotonicity of the whole profile is not guaranteed,
!        because bL(k+1)-bR(k) may still have different sign than
!        qc(i,k+1)-qc(i,k).  This possibility is excluded,
!        after bL and bR are reconciled using WENO procedure.
!
                cff=(dltR-dltL)*Hz_inv3(i,k)
                dltR=dltR-cff*Hz(i,j,k+1)
                dltL=dltL+cff*Hz(i,j,k-1)
                bR(i,k)=qc(i,k)+dltR
                bL(i,k)=qc(i,k)-dltL
                WR(i,k)=(2.0_r8*dltR-dltL)**2
                WL(i,k)=(dltR-2.0_r8*dltL)**2
              END DO
            END DO
            cff=1.0E-14_r8
            DO k=2,N(ng)-2
              DO i=Istr,Iend
                dltL=MAX(cff,WL(i,k  ))
                dltR=MAX(cff,WR(i,k+1))
                bR(i,k)=(dltR*bR(i,k)+dltL*bL(i,k+1))/(dltR+dltL)
                bL(i,k+1)=bR(i,k)
              END DO
            END DO
            DO i=Istr,Iend
              FC(i,N(ng))=0.0_r8            ! NO-flux boundary condition
#if defined LINEAR_CONTINUATION
              bL(i,N(ng))=bR(i,N(ng)-1)
              bR(i,N(ng))=2.0_r8*qc(i,N(ng))-bL(i,N(ng))
#elif defined NEUMANN
              bL(i,N(ng))=bR(i,N(ng)-1)
              bR(i,N(ng))=1.5_r8*qc(i,N(ng))-0.5_r8*bL(i,N(ng))
#else
              bR(i,N(ng))=qc(i,N(ng))       ! default strictly monotonic
              bL(i,N(ng))=qc(i,N(ng))       ! conditions
              bR(i,N(ng)-1)=qc(i,N(ng))
#endif
#if defined LINEAR_CONTINUATION
              bR(i,1)=bL(i,2)
              bL(i,1)=2.0_r8*qc(i,1)-bR(i,1)
#elif defined NEUMANN
              bR(i,1)=bL(i,2)
              bL(i,1)=1.5_r8*qc(i,1)-0.5_r8*bR(i,1)
#else
              bL(i,2)=qc(i,1)               ! bottom grid boxes are
              bR(i,1)=qc(i,1)               ! re-assumed to be
              bL(i,1)=qc(i,1)               ! piecewise constant.
#endif
            END DO
!
!  Apply monotonicity constraint again, since the reconciled interfacial
!  values may cause a non-monotonic behavior of the parabolic segments
!  inside the grid box.
!
            DO k=1,N(ng)
              DO i=Istr,Iend
                dltR=bR(i,k)-qc(i,k)
                dltL=qc(i,k)-bL(i,k)
                cffR=2.0_r8*dltR
                cffL=2.0_r8*dltL
                IF ((dltR*dltL).lt.0.0_r8) THEN
                  dltR=0.0_r8
                  dltL=0.0_r8
                ELSE IF (ABS(dltR).gt.ABS(cffL)) THEN
                  dltR=cffL
                ELSE IF (ABS(dltL).gt.ABS(cffR)) THEN
                  dltL=cffR
                END IF
                bR(i,k)=qc(i,k)+dltR
                bL(i,k)=qc(i,k)-dltL
              END DO
            END DO
!
!  After this moment reconstruction is considered complete. The next
!  stage is to compute vertical advective fluxes, FC. It is expected
!  that sinking may occurs relatively fast, the algorithm is designed
!  to be free of CFL criterion, which is achieved by allowing
!  integration bounds for semi-Lagrangian advective flux to use as
!  many grid boxes in upstream direction as necessary.
!
!  In the two code segments below, WL is the z-coordinate of the
!  departure point for grid box interface z_w with the same indices;
!  FC is the finite volume flux; ksource(:,k) is index of vertical
!  grid box which contains the departure point (restricted by N(ng)).
!  During the search: also add in content of whole grid boxes
!  participating in FC.
!
            cff=dtdays*ABS(Wbio(isink))
            DO k=1,N(ng)
              DO i=Istr,Iend
                FC(i,k-1)=0.0_r8
                WL(i,k)=z_w(i,j,k-1)+cff
                WR(i,k)=Hz(i,j,k)*qc(i,k)
                ksource(i,k)=k
              END DO
            END DO
            DO k=1,N(ng)
              DO ks=k,N(ng)-1
                DO i=Istr,Iend
                  IF (WL(i,k).gt.z_w(i,j,ks)) THEN
                    ksource(i,k)=ks+1
                    FC(i,k-1)=FC(i,k-1)+WR(i,ks)
                  END IF
                END DO
              END DO
            END DO
!
!  Finalize computation of flux: add fractional part.
!
            DO k=1,N(ng)
              DO i=Istr,Iend
                ks=ksource(i,k)
                cu=MIN(1.0_r8,(WL(i,k)-z_w(i,j,ks-1))*Hz_inv(i,ks))
                FC(i,k-1)=FC(i,k-1)+                                    &
     &                    Hz(i,j,ks)*cu*                                &
     &                    (bL(i,ks)+                                    &
     &                     cu*(0.5_r8*(bR(i,ks)-bL(i,ks))-              &
     &                         (1.5_r8-cu)*                             &
     &                         (bR(i,ks)+bL(i,ks)-                      &
     &                          2.0_r8*qc(i,ks))))
              END DO
            END DO
            DO k=1,N(ng)
              DO i=Istr,Iend
                Bio(i,k,ibio)=qc(i,k)+(FC(i,k)-FC(i,k-1))*Hz_inv(i,k)
              END DO
            END DO

          END DO SINK_LOOP
        END DO ITER_LOOP
!
!-----------------------------------------------------------------------
!  Update global tracer variables (m Tunits).
!-----------------------------------------------------------------------
!
        DO itrc=1,NBT
          ibio=idbio(itrc)
          DO k=1,N(ng)
            DO i=Istr,Iend
              t(i,j,k,nnew,ibio)=MAX(t(i,j,k,nnew,ibio)+                &
     &                               (Bio(i,k,ibio)-Bio_bak(i,k,ibio))* &
     &                               Hz(i,j,k),                         &
     &                               0.0_r8)
#ifdef TS_MPDATA
              t(i,j,k,3,ibio)=t(i,j,k,nnew,ibio)*Hz_inv(i,k)
#endif
            END DO
          END DO
        END DO
!        IF (tile == 1) THEN
!           WRITE(6,*) 'jloop end',j,Jstr,Jend
!        END IF
      END DO J_LOOP

      RETURN
      END SUBROUTINE biology_tile

#include "OxygenTransfer.h"
#include "CO2Transfer.h"













 
