      SUBROUTINE read_BioPar (model, inp, out, Lwrite)
!
!svn $Id: ecodynamo_inp.h 1415 2015-07-24 19:42:49Z pedro $
!================================================== Hernan G. Arango ===
!  Copyright (c) 2002-2011 The ROMS/TOMS Group                         !
!    Licensed under a MIT/X style license                              !
!    See License_ROMS.txt                                              !
!=======================================================================
!                                                                      !
!  This routine reads in Ecoynamo ecosystem model input    !
!  parameters. They are specified in input script "ecodynamo.in".    !
!                                                                      !
!=======================================================================
!
      USE mod_param
      USE mod_parallel
      USE mod_biology
#ifdef ECODYNAMO
      !USE analytical_mod
      USE ecodynamocpp_mod
#endif
      USE mod_ncparam
      USE mod_scalars
!
      implicit none
!
!  Imported variable declarations
!
      logical, intent(in) :: Lwrite
      integer, intent(in) :: model, inp, out
!
!  Local variable declarations.
!
      integer :: Npts, Nval
      integer :: iTrcStr, iTrcEnd
      integer :: i, ifield, igrid, itracer, itrc, ng, nline, status

      integer :: decode_line, load_i, load_l, load_lbc, load_r

      logical, dimension(NBT,Ngrids) :: Ltrc

      real(r8), dimension(NBT,Ngrids) :: Rbio

      real(r8), dimension(100) :: Rval

      character (len=40 ) :: KeyWord
      character (len=256) :: line
      character (len=256), dimension(100) :: Cval 

!*******************************************Stuff imported from fennel_input.h***********************************
      logical, dimension(Ngrids) :: Lbio
!****************************************************************************************************************
!-----------------------------------------------------------------------
!  Initialize.
!-----------------------------------------------------------------------
!
      igrid=1                            ! nested grid counter
      itracer=0                          ! LBC tracer counter
      iTrcStr=isTvar(idbio(1))           ! first LBC tracer to process
      iTrcEnd=isTvar(idbio(NBT))         ! last  LBC tracer to process
      nline=0                            ! LBC multi-line counter      
!
!-----------------------------------------------------------------------
!  Read in NPZD biological model (Powell et al., 2006) parameters.
!-----------------------------------------------------------------------
!
#ifdef ANA_BIOLOGY
      IF (.not.allocated(BioIni)) allocate ( BioIni(MT,Ngrids) )
#endif
      DO WHILE (.TRUE.)
        READ (inp,'(a)',ERR=10,END=20) line
        status=decode_line(line, KeyWord, Nval, Cval, Rval)
        IF (status.gt.0) THEN
          SELECT CASE (TRIM(KeyWord))
            CASE ('Lbiology')
              Npts=load_l(Nval, Cval, Ngrids, Lbiology)
            CASE ('BioIter')
              Npts=load_i(Nval, Rval, Ngrids, BioIter)
#ifdef ANA_BIOLOGY
            CASE ('BioIni(iNO3_)')
              Npts=load_r(Nval, Rval, Ngrids, BioIni(iNO3_,1))
            CASE ('BioIni(iPhyt)')
              Npts=load_r(Nval, Rval, Ngrids, BioIni(iPhyt,1))
            CASE ('BioIni(iZoop)')
              Npts=load_r(Nval, Rval, Ngrids, BioIni(iZoop,1))
            CASE ('BioIni(iSDet)')
              Npts=load_r(Nval, Rval, Ngrids, BioIni(iSDet,1))
#endif
            CASE ('PARfrac')
              Npts=load_r(Nval, Rval, Ngrids, PARfrac)
            CASE ('AttSW')
              Npts=load_r(Nval, Rval, Ngrids, AttSW)
            CASE ('AttPhy')
              Npts=load_r(Nval, Rval, Ngrids, AttPhy)
            CASE ('PhyIS')
              Npts=load_r(Nval, Rval, Ngrids, PhyIS)
            CASE ('Vm_NO3')
              Npts=load_r(Nval, Rval, Ngrids, Vm_NO3)
            CASE ('PhyMRD')
              Npts=load_r(Nval, Rval, Ngrids, PhyMRD)
            CASE ('PhyMRN')
              Npts=load_r(Nval, Rval, Ngrids, PhyMRN)
            CASE ('K_NO3')
              Npts=load_r(Nval, Rval, Ngrids, K_NO3)
!********************EcoDynamo added stuff*******************************************  
            CASE ('Beta')
              Npts=load_r(Nval, Rval, Ngrids, Beta)
            CASE ('Chl2C_m') 
              Npts=load_r(Nval, Rval, Ngrids, Chl2C_m)      
            CASE ('Pmax') 
              Npts=load_r(Nval, Rval, Ngrids, Pmax)
            CASE ('Iopt')
              Npts=load_r(Nval, Rval, Ngrids, Iopt) 
            CASE ('ImaxE')
              Npts=load_r(Nval, Rval, Ngrids, ImaxE) 
            CASE ('AEiler')
              Npts=load_r(Nval, Rval, Ngrids, AEiler)
            CASE ('BEiler')
              Npts=load_r(Nval, Rval, Ngrids, BEiler)
            CASE ('CEiler')
              Npts=load_r(Nval, Rval, Ngrids, CEiler)
            CASE ('MaintResp')
              Npts=load_r(Nval, Rval, Ngrids, MaintResp)
            CASE ('RespCoeff')
              Npts=load_r(Nval, Rval, Ngrids, RespCoeff)
            CASE ('DocStressLoss') 
              Npts=load_r(Nval, Rval, Ngrids, DocStressLoss)
            CASE ('RedfieldC') 
              Npts=load_r(Nval, Rval, Ngrids, RedfieldC) 
            CASE ('RedfieldN') 
              Npts=load_r(Nval, Rval, Ngrids, RedfieldN) 
            CASE ('RedfieldP') 
              Npts=load_r(Nval, Rval, Ngrids, RedfieldP) 
            CASE ('TemperatureAugmentationRate') 
              Npts=load_r(Nval, Rval, Ngrids, TemperatureAugmentationRate)
            CASE ('RatioLightDarkRespiration') 
              Npts=load_r(Nval, Rval, Ngrids, RatioLightDarkRespiration)
            CASE ('minNPRatio') 
              Npts=load_r(Nval, Rval, Ngrids, minNPRatio)
            CASE ('maxNPRatio')
              Npts=load_r(Nval, Rval, Ngrids, maxNPRatio)
            CASE ('PMaxUptake') 
              Npts=load_r(Nval, Rval, Ngrids, PMaxUptake)
            CASE ('NMaxUptake')
              Npts=load_r(Nval, Rval, Ngrids, NMaxUptake)
            CASE ('KP')
              Npts=load_r(Nval, Rval, Ngrids, KP)
            CASE ('KNH4') 
              Npts=load_r(Nval, Rval, Ngrids, KNH4)
            CASE ('MinPCellQuota')
              Npts=load_r(Nval, Rval, Ngrids, MinPCellQuota)
            CASE ('MaxPCellQuota')
              Npts=load_r(Nval, Rval, Ngrids, MaxPCellQuota)
            CASE ('MinNCellQuota')
              Npts=load_r(Nval, Rval, Ngrids, MinNCellQuota)
            CASE ('MaxNCellQuota')
              Npts=load_r(Nval, Rval, Ngrids, MaxNCellQuota)
            CASE ('KPInternal')
              Npts=load_r(Nval, Rval, Ngrids, KPInternal)
            CASE ('KNInternal')
              Npts=load_r(Nval, Rval, Ngrids, KNInternal)
            CASE ('CarbonToOxygenProd') 
              Npts=load_r(Nval, Rval, Ngrids, CarbonToOxygenProd)
            CASE ('CarbonToOxygenResp')
              Npts=load_r(Nval, Rval, Ngrids, CarbonToOxygenResp)
            CASE ('TminRespiration')
              Npts=load_r(Nval, Rval, Ngrids, TminRespiration)
            CASE ('TminPhotosynthesis')
              Npts=load_r(Nval, Rval, Ngrids, TminPhotosynthesis)
            CASE ('NitrogenLimitation')
              Npts=load_i(Nval, Rval, Ngrids, NitrogenLimitation)
            CASE ('PhosphorusLimitation')
              Npts=load_i(Nval, Rval, Ngrids, PhosphorusLimitation)
            CASE ('NitriR')
              Npts=load_r(Nval, Rval, Ngrids, NitriR)
            CASE ('kdenit')
              Npts=load_r(Nval, Rval, Ngrids, kdenit)
            CASE ('knitO2') 
              Npts=load_r(Nval, Rval, Ngrids, knitO2)
            CASE ('kdenitO2')
              Npts=load_r(Nval, Rval, Ngrids, kdenitO2)
            CASE ('pCO2air')
              Npts=load_r(Nval, Rval, Ngrids, pCO2air)
            CASE ('kt')
              Npts=load_r(Nval, Rval, Ngrids, kt)
            CASE ('minRateC') 
              Npts=load_r(Nval, Rval, Ngrids, minRateC)
            CASE ('minRateN') 
              Npts=load_r(Nval, Rval, Ngrids, minRateN)
            CASE ('minRateP') 
              Npts=load_r(Nval, Rval, Ngrids, minRateP)
            CASE ('DenitNH4')
              Npts=load_r(Nval, Rval, Ngrids, DenitNH4)
            CASE ('MinToNH4')
              Npts=load_r(Nval, Rval, Ngrids, MinToNH4)
            CASE ('KI')
              Npts=load_r(Nval, Rval, Ngrids, KI)
            CASE ('I0')
              Npts=load_r(Nval, Rval, Ngrids, I0)
            

            CASE ('Chl2C_mDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, Chl2C_mDiatoms)      
            CASE ('PmaxDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, PmaxDiatoms)
            CASE ('IoptDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, IoptDiatoms) 
            CASE ('ImaxEDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, ImaxEDiatoms) 
            CASE ('AEilerDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, AEilerDiatoms)
            CASE ('BEilerDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, BEilerDiatoms)
            CASE ('CEilerDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, CEilerDiatoms)
            CASE ('MaintRespDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, MaintRespDiatoms)
            CASE ('RespCoeffDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, RespCoeffDiatoms)
            CASE ('DocStressLossDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, DocStressLossDiatoms)
            CASE ('RedfieldCDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, RedfieldCDiatoms) 
            CASE ('RedfieldNDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, RedfieldNDiatoms) 
            CASE ('RedfieldPDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, RedfieldPDiatoms) 
            CASE ('TemperatureAugmentationRateDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, TemperatureAugmentationRateDiatoms)
            CASE ('RatioLightDarkRespirationDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, RatioLightDarkRespirationDiatoms)
            CASE ('minNPRatioDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, minNPRatioDiatoms)
            CASE ('maxNPRatioDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, maxNPRatioDiatoms)
            CASE ('PMaxUptakeDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, PMaxUptakeDiatoms)
            CASE ('NMaxUptakeDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, NMaxUptakeDiatoms)
            CASE ('SiMaxUptake')
              Npts=load_r(Nval, Rval, Ngrids, SiMaxUptake)
            CASE ('KPDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, KPDiatoms)
            CASE ('K_NO3Diatoms') 
              Npts=load_r(Nval, Rval, Ngrids, K_NO3Diatoms)
            CASE ('KNH4Diatoms') 
              Npts=load_r(Nval, Rval, Ngrids, KNH4Diatoms)
            CASE ('MinPCellQuotaDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, MinPCellQuotaDiatoms)
            CASE ('MaxPCellQuotaDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, MaxPCellQuotaDiatoms)
            CASE ('MinNCellQuotaDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, MinNCellQuotaDiatoms)
            CASE ('MaxNCellQuotaDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, MaxNCellQuotaDiatoms)
            CASE ('KPInternalDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, KPInternalDiatoms)
            CASE ('KNInternalDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, KNInternalDiatoms)
            CASE ('KSiInternal')
              Npts=load_r(Nval, Rval, Ngrids, KSiInternal)
            CASE ('CarbonToOxygenProdDiatoms') 
              Npts=load_r(Nval, Rval, Ngrids, CarbonToOxygenProdDiatoms)
            CASE ('CarbonToOxygenRespDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, CarbonToOxygenRespDiatoms)
            CASE ('TminRespirationDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, TminRespirationDiatoms)
            CASE ('TminPhotosynthesisDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, TminPhotosynthesisDiatoms)
            CASE ('NitrogenLimitationDiatoms')
              Npts=load_i(Nval, Rval, Ngrids, NitrogenLimitationDiatoms)
            CASE ('PhosphorusLimitationDiatoms')
              Npts=load_i(Nval, Rval, Ngrids, PhosphorusLimitationDiatoms)
            CASE ('SilicaLimitation')
              Npts=load_i(Nval, Rval, Ngrids, SilicaLimitation)
            CASE ('PhyISDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, PhyISDiatoms) 
            CASE ('BetaDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, BetaDiatoms) 
            CASE ('PhyMRDDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, PhyMRDDiatoms) 
            CASE ('PhyMRNDiatoms')
              Npts=load_r(Nval, Rval, Ngrids, PhyMRNDiatoms)
            CASE ('Vm_NO3Diatoms')
              Npts=load_r(Nval, Rval, Ngrids, Vm_NO3Diatoms) 
            CASE ('MaxSiCellQuota')
              Npts=load_r(Nval, Rval, Ngrids, MaxSiCellQuota) 
            CASE ('minNSiRatio')
              Npts=load_r(Nval, Rval, Ngrids, minNSiRatio) 
            CASE ('KSi')
              Npts=load_r(Nval, Rval, Ngrids, KSi) 
            CASE ('RedfieldSi')
              Npts=load_r(Nval, Rval, Ngrids, RedfieldSi)  
!******************************************************************************       
            CASE ('Ivlev')
              Npts=load_r(Nval, Rval, Ngrids, Ivlev)
            CASE ('ZooGR')
              Npts=load_r(Nval, Rval, Ngrids, ZooGR)
            CASE ('ZooEED')
              Npts=load_r(Nval, Rval, Ngrids, ZooEED)
            CASE ('ZooEEN')
              Npts=load_r(Nval, Rval, Ngrids, ZooEEN)
            CASE ('ZooMRD')
              Npts=load_r(Nval, Rval, Ngrids, ZooMRD)
            CASE ('ZooMRN')
              Npts=load_r(Nval, Rval, Ngrids, ZooMRN)
            CASE ('DetRR')
              Npts=load_r(Nval, Rval, Ngrids, DetRR)
            CASE ('wPhy')
              Npts=load_r(Nval, Rval, Ngrids, wPhy)
            CASE ('wDet')
              Npts=load_r(Nval, Rval, Ngrids, wDet)
            CASE ('TNU2')
              Npts=load_r(Nval, Rval, NBT*Ngrids, Rbio)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idbio(itrc)
                  nl_tnu2(i,ng)=Rbio(itrc,ng)
                END DO
              END DO
            CASE ('TNU4')
              Npts=load_r(Nval, Rval, NBT*Ngrids, Rbio)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idbio(itrc)
                  nl_tnu4(i,ng)=Rbio(itrc,ng)
                END DO
              END DO
            CASE ('ad_TNU2')
              Npts=load_r(Nval, Rval, NBT*Ngrids, Rbio)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idbio(itrc)
                  ad_tnu2(i,ng)=Rbio(itrc,ng)
                  tl_tnu2(i,ng)=Rbio(itrc,ng)
                END DO
              END DO
            CASE ('ad_TNU4')
              Npts=load_r(Nval, Rval, NBT*Ngrids, Rbio)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idbio(itrc)
                  ad_tnu4(i,ng)=Rbio(itrc,ng)
                  ad_tnu4(i,ng)=Rbio(itrc,ng)
                END DO
              END DO
            CASE ('AKT_BAK')
              Npts=load_r(Nval, Rval, NBT*Ngrids, Rbio)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idbio(itrc)
                  Akt_bak(i,ng)=Rbio(itrc,ng)
                END DO
              END DO
            CASE ('ad_AKT_fac')
              Npts=load_r(Nval, Rval, NBT*Ngrids, Rbio)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idbio(itrc)
                  ad_Akt_fac(i,ng)=Rbio(itrc,ng)
                  tl_Akt_fac(i,ng)=Rbio(itrc,ng)
                END DO
              END DO
            CASE ('TNUDG')
              Npts=load_r(Nval, Rval, NBT*Ngrids, Rbio)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idbio(itrc)
                  Tnudg(i,ng)=Rbio(itrc,ng)
                END DO
              END DO
            CASE ('LBC(isTvar)')
              IF (itracer.lt.NBT) THEN
                itracer=itracer+1
              ELSE
                itracer=1                      ! next nested grid
              END IF
              ifield=isTvar(idbio(itracer))
              Npts=load_lbc(Nval, Cval, line, nline, ifield, igrid,     &
     &                      iTrcStr, iTrcEnd, LBC)
#if defined ADJOINT || defined TANGENT || defined TL_IOMS
            CASE ('ad_LBC(isTvar)')
              IF (itracer.lt.NBT) THEN
                itracer=itracer+1
              ELSE
                itracer=1                      ! next nested grid
              END IF
              ifield=isTvar(idbio(itracer))
              Npts=load_lbc(Nval, Cval, line, nline, ifield, igrid,     &
     &                      iTrcStr, iTrcEnd, ad_LBC)
#endif
#ifdef TCLIMATOLOGY
            CASE ('LtracerCLM')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idbio(itrc)
                  LtracerCLM(i,ng)=Ltrc(itrc,ng)
                END DO
              END DO
#endif
#ifdef TS_PSOURCE
            CASE ('LtracerSrc')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idbio(itrc)
                  LtracerSrc(i,ng)=Ltrc(itrc,ng)
                END DO
              END DO
#endif
          CASE ('Hout(idTvar)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idTvar(idbio(itrc))
                  IF (i.eq.0) THEN
                    IF (Master) WRITE (out,30)                          &
     &                                'idTvar(idbio(', itrc, '))'
                    exit_flag=5
                    RETURN
                  END IF
                  Hout(i,ng)=Ltrc(itrc,ng)
                END DO
              END DO
            CASE ('Hout(idTsur)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idTsur(idbio(itrc))
                  IF (i.eq.0) THEN
                    IF (Master) WRITE (out,30)                          &
     &                                'idTsur(idbio(', itrc, '))'
                    exit_flag=5
                    RETURN
                  END IF
                  Hout(i,ng)=Ltrc(itrc,ng)
                END DO
              END DO
#if defined AVERAGES    || \
   (defined AD_AVERAGES && defined ADJOINT) || \
   (defined RP_AVERAGES && defined TL_IOMS) || \
   (defined TL_AVERAGES && defined TANGENT)
            CASE ('Aout(idTvar)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO itrc=1,NBT
                  i=idTvar(idbio(itrc))
                  Aout(i,ng)=Ltrc(itrc,ng)
                END DO
              END DO
#endif
#ifdef DIAGNOSTICS_TS
             CASE ('Dout(iTrate)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO i=1,NBT
                  itrc=idbio(i)
                  Dout(idDtrc(itrc,iTrate),ng)=Ltrc(i,ng)
                END DO
              END DO
            CASE ('Dout(iThadv)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO i=1,NBT
                  itrc=idbio(i)
                  Dout(idDtrc(itrc,iThadv),ng)=Ltrc(i,ng)
                END DO
              END DO
            CASE ('Dout(iTxadv)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO i=1,NBT
                  itrc=idbio(i)
                  Dout(idDtrc(itrc,iTxadv),ng)=Ltrc(i,ng)
                END DO
              END DO
            CASE ('Dout(iTyadv)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO i=1,NBT
                  itrc=idbio(i)
                  Dout(idDtrc(itrc,iTyadv),ng)=Ltrc(i,ng)
                END DO
              END DO
            CASE ('Dout(iTvadv)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO i=1,NBT
                  itrc=idbio(i)
                  Dout(idDtrc(itrc,iTvadv),ng)=Ltrc(i,ng)
                END DO
              END DO
# if defined TS_DIF2 || defined TS_DIF4
            CASE ('Dout(iThdif)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO i=1,NBT
                  itrc=idbio(i)
                  Dout(idDtrc(itrc,iThdif),ng)=Ltrc(i,ng)
                END DO
              END DO
            CASE ('Dout(iTxdif)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO i=1,NBT
                  itrc=idbio(i)
                  Dout(idDtrc(itrc,iTxdif),ng)=Ltrc(i,ng)
                END DO
              END DO
            CASE ('Dout(iTydif)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO i=1,NBT
                  itrc=idbio(i)
                  Dout(idDtrc(itrc,iTydif),ng)=Ltrc(i,ng)
                END DO
              END DO
#  if defined MIX_GEO_TS || defined MIX_ISO_TS
            CASE ('Dout(iTsdif)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO i=1,NBT
                  itrc=idbio(i)
                  Dout(idDtrc(itrc,iTsdif),ng)=Ltrc(i,ng)
                END DO
              END DO
#  endif
# endif
            CASE ('Dout(iTvdif)')
              Npts=load_l(Nval, Cval, NBT*Ngrids, Ltrc)
              DO ng=1,Ngrids
                DO i=1,NBT
                  itrc=idbio(i)
                  Dout(idDtrc(itrc,iTvdif),ng)=Ltrc(i,ng)
                END DO
              END DO
#  endif
!******************Imported from Fennel model***********************************************************
#ifdef DIAGNOSTICS_BIO
# ifdef CARBON
            CASE ('Dout(iCOfx)')
            IF (iDbio2(iCOfx).eq.0) THEN
              IF (Master) WRITE (out,40) 'iDbio2(iCOfx)'
              exit_flag=5
              RETURN
            END IF
            Npts=load_l(Nval, Cval, Ngrids, Lbio)
            i=iDbio2(iCOfx)
            DO ng=1,Ngrids
              Dout(i,ng)=Lbio(ng)
            END DO
# endif
# ifdef DENITRIFICATION
            CASE ('Dout(iDNIT)')
            IF (iDbio2(iDNIT).eq.0) THEN
              IF (Master) WRITE (out,40) 'iDbio2(iDNIT)'
              exit_flag=5
              RETURN
            END IF
            Npts=load_l(Nval, Cval, Ngrids, Lbio)
            i=iDbio2(iDNIT)
            DO ng=1,Ngrids
              Dout(i,ng)=Lbio(ng)
            END DO
# endif
# ifdef CARBON
            CASE ('Dout(ipCO2)')
            IF (iDbio2(ipCO2).eq.0) THEN
              IF (Master) WRITE (out,40) 'iDbio2(ipCO2)'
              exit_flag=5
              RETURN
            END IF
            Npts=load_l(Nval, Cval, Ngrids, Lbio)
            i=iDbio2(ipCO2)
            DO ng=1,Ngrids
              Dout(i,ng)=Lbio(ng)
            END DO
# endif
# ifdef OXYGEN
            CASE ('Dout(iO2fx)')
            IF (iDbio2(iO2fx).eq.0) THEN
              IF (Master) WRITE (out,40) 'iDbio2(iO2fx)'
              exit_flag=5
              RETURN
            END IF
            Npts=load_l(Nval, Cval, Ngrids, Lbio)
            i=iDbio2(iO2fx)
            DO ng=1,Ngrids
              Dout(i,ng)=Lbio(ng)
            END DO
# endif
            CASE ('Dout(iPPro)')
            IF (iDbio3(iPPro).eq.0) THEN
              IF (Master) WRITE (out,40) 'iDbio3(iPPro)'
              exit_flag=5
              RETURN
            END IF
            Npts=load_l(Nval, Cval, Ngrids, Lbio)
            i=iDbio3(iPPro)
            DO ng=1,Ngrids
              Dout(i,ng)=Lbio(ng)
            END DO

            CASE ('Dout(iNO3u)') 
            IF (iDbio3(iNO3u).eq.0) THEN
              IF (Master) WRITE (out,40) 'iDbio3(iNO3u)'
              exit_flag=5
              RETURN
            END IF
            Npts=load_l(Nval, Cval, Ngrids, Lbio)
            i=iDbio3(iNO3u)
            DO ng=1,Ngrids
              Dout(i,ng)=Lbio(ng)
            END DO
#endif
!***************************************************************************************************************
          END SELECT
        END IF
      END DO
  10  IF (Master) WRITE (out,40) line
      exit_flag=4
      RETURN
  20  CONTINUE
!
!-----------------------------------------------------------------------
!  Report input parameters.
!-----------------------------------------------------------------------
!
      IF (Lwrite) THEN
        DO ng=1,Ngrids
          IF (Lbiology(ng)) THEN
            WRITE (out,50) ng
            WRITE (out,60) BioIter(ng), 'BioIter',                      &
     &            'Number of iterations for nonlinear convergence.'
#ifdef ANA_BIOLOGY
            WRITE (out,70) BioIni(iNO3_,ng), 'BioIni(iNO3_)',           &
     &            'Nitrate initial concentration (mmol/m3).'
            WRITE (out,70) BioIni(iPhyt,ng), 'BioIni(iPhyt)',           &
     &            'Phytoplankton initial concentration (mmol/m3).'
            WRITE (out,70) BioIni(iZoop,ng), 'BioIni(iZoop)',           &
     &            'Zooplankton initial concentration (mmol/m3).'
            WRITE (out,70) BioIni(iSDet,ng), 'BioIni(iSDet)',           &
     &            'Small detritus initial concentration (mmol/m3).'
#endif
            WRITE (out,80) PARfrac(ng), 'PARfrac',                      &
     &            'Fraction of shortwave radiation that is',            &
     &            'photosynthetically active (nondimensional).'
            WRITE (out,70) AttSW(ng), 'AttSW',                          &
     &            'Light attenuation of seawater (m-1).'
            WRITE (out,70) AttPhy(ng), 'AttPhy',                        &
     &            'Light attenuation by phytoplankton (m2/mmole_N).'
            WRITE (out,80) PhyIS(ng), 'PhyIS',                          &
     &            'Phytoplankton growth, initial slope of P-I curve',   &
     &            '(mg C mg C-1 h-1 microM photons m2 s).'
            WRITE (out,80) Beta(ng), 'Beta',                            &
     &            'Phytoplankton growth, negative slope of P-I curve ', &
     &            '(mg C mg C-1 h-1 microM photons m2 s).'
            WRITE (out,70) Vm_NO3(ng), 'Vm_NO3',                        &
     &            'Nitrate upatake rate (day-1).'
            WRITE (out,70) PhyMRD(ng), 'PhyMRD',                        &
     &            'Phytoplankton mortality rate to Detritus (day-1)'
            WRITE (out,70) PhyMRN(ng), 'PhyMRN',                        &
     &            'Phytoplankton mortality rate to Nitrogen (day-1)'
            WRITE (out,80) K_NO3(ng), 'K_NO3',                          &
     &            'Inverse half-saturation for phytoplankton NO3',      &
     &            'uptake (1/(mmol m-3)).'
!****************************EcoDynamo added stuff***************************
            !WRITE (*,*) 'Chl2C_m',Chl2C_m(1,ng),Chl2C_m(2,ng)           &
            !DO i=1,Phyto_groups
            ! WRITE (out,80) Chl2C_m(i,ng), 'Chl2C_m',      &
            WRITE (out,80) Chl2C_m(ng), 'Chl2C_m',                       &
     &            'Maximum chlorophyll to carbon ratio [mg Chl/ mg C].' 
     !       END DO 
            WRITE (out,80) Pmax(ng), 'Pmax',                            &
     &            'Maximum photosynthetic rate [1/h].'      
            WRITE (out,80) Iopt(ng), 'Iopt',                            &
     &            'Optimal light intensity [microEisntein/m2/s].'      
            WRITE (out,80) ImaxE(ng), 'ImaxE',                            &
     &            'Saturation light intensity [microEisntein/m2/s].'
            WRITE (out,80) AEiler(ng), 'AEiler',                        &
     &            'Parameter of the Eilers & Peeters equation',           &               
     &             '[microEinstein/m2/s].'
            WRITE (out,80) BEiler(ng), 'BEiler',                        &
     &            'Parameter of the Eilers & Peeters equation',           &               
     &             '[microEinstein/m2/s].'
            WRITE (out,80) CEiler(ng), 'CEiler',                        &
     &            'Parameter of the Eilers & Peeters equation',           &               
     &             '[microEinstein/m2/s].'
            WRITE (out,80) MaintResp(ng), 'MaintResp',                  &
     &            'Maintenance respiration [mmol O2/mg Chl/h].'
            WRITE (out,80) RespCoeff(ng), 'RespCoeff',                  &
     &            'Respiration coefficient [nondimensional].'
            WRITE (out,80) DocStressLoss(ng), 'DocStressLoss',          &
     &            'Fraction of GPP lost as dissolved organics',           &
     &             'under nutriebnt stress [nondimensional].'
            WRITE (out,80) RedfieldC(ng), 'RedfieldC',                  &
     &            'Redfield "default" mass of C per mass of P',           & 
     &             '[gC/gP].'
            WRITE (out,80) RedfieldN(ng), 'RedfieldN',                  &
     &            'Redfield "default" mass of N per mass of P',           & 
     &             '[gN/gP].'
            WRITE (out,80) RedfieldP(ng), 'RedfieldP',                  &
     &            'Redfield "default" mass of P.'
            WRITE (out,80) TemperatureAugmentationRate(ng),             & 
     &            'TemperatureAugmentationRate',                        &
     &            'Exponent of the temperature limiting function',        &
     &             '[1/degrees].'
            WRITE (out,80) RatioLightDarkRespiration(ng),               & 
                  'RatioLightDarkRespiration',                          &
     &            'Ratio between respiration in the light',               &
     &             'and respiration in the darkness[1/degrees].'  
            WRITE (out,80) minNPRatio(ng), 'minNPRatio',                &
     &            'Minimal N:P cell ratio [mgN/mgP].'
            WRITE (out,80) maxNPRatio(ng), 'maxNPRatio',                &
     &            'Maximal N:P cell ratio [mgN/mgP].'
            WRITE (out,80) PMaxUptake(ng), 'PMaxUptake',                &
     &            'Maximal phosphorus uptake [1/h].'
            WRITE (out,80) NMaxUptake(ng), 'NMaxUptake',                &
     &            'Maximal nitrogen uptake [1/h].'  
            WRITE (out,80) KP(ng), 'KP',                                    &
     &            'Half-saturation constant for P uptake [microM].'     
            WRITE (out,80) KNH4(ng), 'KNH4',                                  &
     &            'Half-saturation constant for ammonia uptake',          & 
     &             '[microM].'       
            WRITE (out,80) MinPCellQuota(ng), 'MinPCellQuota',              &
     &            'Minimal P cell contents [mgP/mgC].'
            WRITE (out,80) MaxPCellQuota(ng), 'MaxPCellQuota',              &
     &            'Maximal P cell contents [mgP/mgC].'
            WRITE (out,80) MinNCellQuota(ng), 'MinNCellQuota',              &
     &            'Minimal N cell contents [mgN/mgC].'
            WRITE (out,80) MaxNCellQuota(ng), 'MaxNCellQuota',              &
     &            'Maximal N cell contents [mgN/mgC].'            
            WRITE (out,80) KPInternal(ng), 'KPInternal',                    &
     &            'Half saturation constant for internal P limitation',   &
     &             '[mgP/mgC].'    
            WRITE (out,80) KNInternal(ng), 'KNInternal',                    &
     &            'Half saturation constant for internal N limitation' ,  &
     &             '[mgN/mgC].'  
            WRITE (out,80) CarbonToOxygenProd(ng), 'CarbonToOxygenProd',    &
     &            'Mass ratio between carbon fixed and',                  & 
     &             'oxygen released during photosynthesis [mgC/mgO2].'
            WRITE (out,80) CarbonToOxygenResp(ng), 'CarbonToOxygenResp',    &
     &            'Mass ratio between carbon released and',               & 
     &             'consumed during photosynthesis [mgC/mgO2].'
            WRITE (out,80) TminRespiration(ng), 'TminRespiration',          &
     &            'Parameter of the exponent of the temperature',         & 
     &             'limiting function for respiration [1/degrees].'
            WRITE (out,80) TminPhotosynthesis(ng), 'TminPhotosynthesis',    &
     &            'Parameter of the exponent of the temperature',         & 
     &             'limiting function for photosynthesis [1/degrees] .'
	    WRITE (out,*) NitrogenLimitation(ng), 'NitrogenLimitation',    &
     &            'Option for not including (0) or',                      & 
     &             'including (1)N limitation [nondimensional].'      
	    WRITE (out,*) PhosphorusLimitation(ng), 'PhosphorusLimitation',&
     &            'Option for not including (0) or',                      & 
     &             'including (1)P limitation [nondimensional].'   
            
     
            WRITE (out,80) Chl2C_mDiatoms(ng), 'Chl2C_mDiatoms',        &
     &            'Maximum chlorophyll to carbon ratio [mg Chl/ mg C].' 
            WRITE (out,80) PmaxDiatoms(ng), 'PmaxDiatoms',                &
     &            'Maximum photosynthetic rate [1/h].'      
            WRITE (out,80) IoptDiatoms(ng), 'IoptDiatoms',              &
     &            'Optimal light intensity [microEisntein/m2/s].'      
            WRITE (out,80) ImaxEDiatoms(ng), 'ImaxEDiatoms',              &
     &            'Saturation light intensity [microEisntein/m2/s].'
            WRITE (out,80) AEilerDiatoms(ng), 'AEilerDiatoms',          &
     &            'Parameter of the Eilers & Peeters equation',           &               
     &             '[microEinstein/m2/s].'
            WRITE (out,80) BEilerDiatoms(ng), 'BEilerDiatoms',          &
     &            'Parameter of the Eilers & Peeters equation',           &               
     &             '[microEinstein/m2/s].'
            WRITE (out,80) CEilerDiatoms(ng), 'CEilerDiatoms',          &
     &            'Parameter of the Eilers & Peeters equation',           &               
     &             '[microEinstein/m2/s].'
            WRITE (out,80) MaintRespDiatoms(ng), 'MaintRespDiatoms',    &
     &            'Maintenance respiration [mmol O2/mg Chl/h].'
            WRITE (out,80) RespCoeffDiatoms(ng), 'RespCoeffDiatoms',      &
     &            'Respiration coefficient [nondimensional].'
            WRITE (out,80) DocStressLossDiatoms(ng),                    &
     &            'DocStressLossDiatoms',                                 &
     &            'Fraction of GPP lost as dissolved organics',         &
     &             'under nutriebnt stress [nondimensional].'
            WRITE (out,80) RedfieldCDiatoms(ng), 'RedfieldCDiatoms',    &
     &            'Redfield "default" mass of C per mass of P',           & 
     &             '[gC/gP].'
            WRITE (out,80) RedfieldNDiatoms(ng), 'RedfieldNDiatoms',    &
     &            'Redfield "default" mass of N per mass of P',           & 
     &             '[gN/gP].'
            WRITE (out,80) RedfieldPDiatoms(ng), 'RedfieldPDiatoms',    &
     &            'Redfield "default" mass of P.'
            WRITE (out,80) RedfieldSi(ng), 'RedfieldSi',                &
     &            'Redfield "default" mass of Si.'
            WRITE (out,80) TemperatureAugmentationRateDiatoms(ng),        & 
     &            'TemperatureAugmentationRateDiatoms',                 &
     &            'Exponent of the temperature limiting function',        &
     &             '[1/degrees].'
            WRITE (out,80) RatioLightDarkRespirationDiatoms(ng),        & 
     &            'RatioLightDarkRespirationDiatoms',                     &
     &            'Ratio between respiration in the light',             &
     &             'and respiration in the darkness[1/degrees].'  
            WRITE (out,80) minNPRatioDiatoms(ng), 'minNPRatioDiatoms',    &
     &            'Minimal N:P cell ratio [mgN/mgP].'
            WRITE (out,80) maxNPRatioDiatoms(ng), 'maxNPRatioDiatoms',  &
     &            'Maximal N:P cell ratio [mgN/mgP].'
            WRITE (out,80) PMaxUptakeDiatoms(ng), 'PMaxUptakeDiatoms',    &
     &            'Maximal phosphorus uptake [1/h].'
            WRITE (out,80) NMaxUptakeDiatoms(ng), 'NMaxUptakeDiatoms',  &
     &            'Maximal nitrogen uptake [1/h].'  
            WRITE (out,80) SiMaxUptake(ng), 'SiMaxUptake',                &
     &            'Maximal silica uptake [1/h].'   
            WRITE (out,80) KPDiatoms(ng), 'KPDiatoms',                    &
     &            'Half-saturation constant for P uptake [microM].'   
            WRITE (out,80) K_NO3Diatoms(ng), 'K_NO3Diatoms',            &
     &            'Half-saturation constant for ammonia uptake',          & 
     &             '[microM].'     
            WRITE (out,80) KNH4Diatoms(ng), 'KNH4Diatoms',              &
     &            'Half-saturation constant for ammonia uptake',          & 
     &             '[microM].'       
            WRITE (out,80) MinPCellQuotaDiatoms(ng),                    &   
     &            'MinPCellQuotaDiatoms',                                 &
     &            'Minimal P cell contents [mgP/mgC].'
            WRITE (out,80) MaxPCellQuotaDiatoms(ng),                    &
     &            'MaxPCellQuotaDiatoms',                                 &
     &            'Maximal P cell contents [mgP/mgC].'
            WRITE (out,80) MinNCellQuotaDiatoms(ng),                    & 
     &            'MinNCellQuotaDiatoms',                                 &
     &            'Minimal N cell contents [mgN/mgC].'
            WRITE (out,80) MaxNCellQuotaDiatoms(ng),                    &
     &            'MaxNCellQuotaDiatoms',                                 &
     &            'Maximal N cell contents [mgN/mgC].'            
            WRITE (out,80) KPInternalDiatoms(ng), 'KPInternalDiatoms',  &
     &            'Half saturation constant for internal P limitation',   &
     &             '[mgP/mgC].'    
            WRITE (out,80) KNInternalDiatoms(ng), 'KNInternalDiatoms',  &
     &            'Half saturation constant for internal N limitation',   &
     &             '[mgN/mgC].'  
            WRITE (out,80) KSiInternal(ng), 'KSiInternal',              &
     &            'Half saturation constant for internal Si limitation',  &
     &             '[mgSi/mgC].' 
            WRITE (out,80) KSi(ng), 'KSi',                              &
     &            'Half saturation constant for Si utake',                &
     &             '[microM].' 
            WRITE (out,80) CarbonToOxygenProdDiatoms(ng),               &
     &             'CarbonToOxygenProdDiatoms',                           &
     &            'Mass ratio between carbon fixed and',                & 
     &             'oxygen released during photosynthesis [mgC/mgO2].'
            WRITE (out,80) CarbonToOxygenRespDiatoms(ng),               &
     &            'CarbonToOxygenRespDiatoms',                            &
     &            'Mass ratio between carbon released and',             & 
     &             'consumed during photosynthesis [mgC/mgO2].'
            WRITE (out,80) TminRespirationDiatoms(ng),                  &
     &            'TminRespirationDiatoms',                               &
     &            'Parameter of the exponent of the temperature',       & 
     &             'limiting function for respiration [1/degrees].'
            WRITE (out,80) TminPhotosynthesisDiatoms(ng),               &
     &            'TminPhotosynthesisDiatoms',    &
     &            'Parameter of the exponent of the temperature',         & 
     &             'limiting function for photosynthesis [1/degrees] .'
	    WRITE (out,*) NitrogenLimitationDiatoms(ng),                &
                  'NitrogenLimitationDiatoms',                            &
     &            'Option for not including (0) or',                    & 
     &             'including (1)N limitation [nondimensional].'      
	    WRITE (out,*) PhosphorusLimitationDiatoms(ng),              &
     &            'PhosphorusLimitationDiatoms',                          &
     &            'Option for not including (0) or',                    & 
     &             'including (1)P limitation [nondimensional].'   
            WRITE (out,*) SilicaLimitation(ng),                           &
     &            'SilicaLimitation',                                   &
     &            'Option for not including (0) or',                      & 
     &             'including (1)Si limitation [nondimensional].' 
            WRITE (out,80) MaxSiCellQuota(ng),                          &
     &            'MaxSiCellQuota',                                       &
     &            'Maximal silica quota.'      
            WRITE (out,80) minNSiRatio(ng),                             &
     &            'minNSiRatio',                                          &
     &            'Minimal N:Si cell ratio [mgN/mgSi].'                            
            WRITE (out,80) PhyISDiatoms(ng),                             &
     &            'PhyISDiatoms',                                         &
     &            'Phytoplankton growth, initial slope of P-I curve',   &
     &            '(mg C mg C-1 h-1 microM photons m2 s).'
            WRITE (out,80) BetaDiatoms(ng),                             &
     &            'BetaDiatoms',                                         &
     &            'Phytoplankton growth, inegative slope of P-I curve', &
     &            '(mg C mg C-1 h-1 microM photons m2 s).'
            WRITE (out,80) PhyMRDDiatoms(ng),                            &
     &             'PhyMRDDiatoms',                                       &
     &             'Phytoplankton mortality rate to Detritus (day-1)'
            WRITE (out,80) PhyMRNDiatoms(ng),                            &
     &             'PhyMRNDiatoms',                                       &
     &             'Phytoplankton mortality rate to Nitrogen (day-1)'
            WRITE (out,70) Vm_NO3Diatoms(ng), 'Vm_NO3Diatoms',          &
     &            'Nitrate upatake rate (day-1).'
            WRITE (out,80) NitriR(ng), 'NitriR',                           &
     &            'Nitrification rate: oxidation of NH4 to NO3',          &
     &             '[1/d].'   
            WRITE (out,80) kdenit(ng), 'kdenit',                           &
     &            'Denitrification rate: reduction of NO3 to NH4',        &
     &             'and N2 [1/d].'   
            WRITE (out,80) knitO2(ng), 'knitO2',                           &
     &            'Half-saturation constant for oxygen limitation',       &
     &             'of nitrification,[mg O2 L-1].'                                                
            WRITE (out,*) kdenitO2(ng), 'kdenitO2',                       &
     &            'Parameter of the oxygen inhibition',                   &
     &            'denitrification function',                          &
     &            '[mg O2 L-1].' 
            WRITE (out,80) pCO2air(ng), 'pCO2air',                         &
     &            'CO2 partial pressure in the air',                   &
     &            '[parts per million by volume].'    
            WRITE (out,80) kt(ng), 'kt',                                   &
     &            'Temperature limiting factor',                       &
     &            'for mineralization, [Degrees centigrades-1].'                     
            WRITE (out,80) minRateC(ng), 'minRateC',                       &
     &            'Mineralization rate of organic matter',             &
     &            '[days-1].'     
            WRITE (out,80) minRateN(ng), 'minRateN',                       &
     &            'Mineralization rate of organic matter',             &
     &            '[days-1].'
            WRITE (out,80) minRateP(ng), 'minRateP',                       &
     &            'Mineralization rate of organic matter',             &
     &            '[days-1].' 
            WRITE (out,80) DenitNH4(ng), 'DenitNH4',                       &
     &            'Proportion of NH4 from denitrification',            &
     &            '[Dimensionless].'   
            WRITE (out,80) MinToNH4(ng),'MinToNH4',                        &
     &            'Fraction of dissolved organic nitrogen',&
     &            'mineralized to ammonia, [Dimensionless].'                      
            WRITE (out,*) KI(ng),'KI',                                    &
     &            'Light intensity at which the ',                     &
     &            'inhibition of nitrification is half-saturated',        &
     &            '[W m-2].'   
            WRITE (out,80) I0(ng),'I0',                                    &
     &            'Light intensity threshold for',                     &
     &            'light-inhibition of nitrification, [W m-2].'   
!******************************************************************************
            WRITE (out,80) Ivlev(ng), 'Ivlev',                          &
     &            'Ivlev constant for zooplankton grazing',             &
     &            '(nondimensional).'
            WRITE (out,70) ZooGR(ng), 'ZooGR',                          &
     &            'Zooplankton maximum growth rate (day-1).'
            WRITE (out,80) ZooEED(ng), 'ZooEED',                        &
     &            'Zooplankton excretion efficiency to Detritus',       &
     &            'pool (nondimensional).'
            WRITE (out,80) ZooEEN(ng), 'ZooEEN',                        &
     &            'Zooplankton excretion efficiency to Nitrogen',       &
     &            'pool (nondimensional).'
            WRITE (out,70) ZooMRD(ng), 'ZooMRD',                        &
     &            'Zooplankton mortality rate to Detritus (day-1).'
            WRITE (out,70) ZooMRN(ng), 'ZooMRN',                        &
     &            'Zooplankton mortality rate to Nitrogen (day-1).'
            WRITE (out,70) DetRR(ng), 'DetRR',                          &
     &            'Detritus remineralization rate (day-1).'
            WRITE (out,70) wPhy(ng), 'wPhy',                            &
     &            'Phytoplankton sinking rate (m/day).'
            WRITE (out,70) wDet(ng), 'wDet',                            &
     &            'Detrital sinking rate (m/day).'
#ifdef TS_DIF2
            DO itrc=1,NBT
              i=idbio(itrc)
              WRITE (out,90) nl_tnu2(i,ng), 'nl_tnu2', i,               &
     &              'NLM Horizontal, harmonic mixing coefficient',      &
     &              '(m2/s) for tracer ', i, TRIM(Vname(1,idTvar(i)))
# ifdef ADJOINT
              WRITE (out,90) ad_tnu2(i,ng), 'ad_tnu2', i,               &
     &              'ADM Horizontal, harmonic mixing coefficient',      &
     &              '(m2/s) for tracer ', i, TRIM(Vname(1,idTvar(i)))
# endif
# if defined TANGENT || defined TL_IOMS
              WRITE (out,90) tl_tnu2(i,ng), 'tl_tnu2', i,               &
     &              'TLM Horizontal, harmonic mixing coefficient',      &
     &              '(m2/s) for tracer ', i, TRIM(Vname(1,idTvar(i)))
# endif
            END DO
#endif
#ifdef TS_DIF4
            DO itrc=1,NBT
              i=idbio(itrc)
              WRITE (out,90) nl_tnu4(i,ng), 'nl_tnu4', i,               &
     &              'NLM Horizontal, biharmonic mixing coefficient',    &
     &              '(m4/s) for tracer ', i, TRIM(Vname(1,idTvar(i)))
# ifdef ADJOINT
              WRITE (out,90) ad_tnu4(i,ng), 'ad_tnu4', i,               &
     &              'ADM Horizontal, biharmonic mixing coefficient',    &
     &              '(m4/s) for tracer ', i, TRIM(Vname(1,idTvar(i)))
# endif
# if defined TANGENT || defined TL_IOMS
              WRITE (out,90) tl_tnu4(i,ng), 'tl_tnu4', i,               &
     &              'TLM Horizontal, biharmonic mixing coefficient',    &
     &              '(m4/s) for tracer ', i, TRIM(Vname(1,idTvar(i)))
# endif
            END DO
#endif
            DO itrc=1,NBT
              i=idbio(itrc)
              WRITE(out,90) Akt_bak(i,ng), 'Akt_bak', i,                &
     &             'Background vertical mixing coefficient (m2/s)',     &
     &             'for tracer ', i, TRIM(Vname(1,idTvar(i)))
            END DO
#ifdef FORWARD_MIXING
            DO itrc=1,NBT
              i=idbio(itrc)
# ifdef ADJOINT
              WRITE (out,90) ad_Akt_fac(i,ng), 'ad_Akt_fac', i,         &
     &              'ADM basic state vertical mixing scale factor',     &
     &              'for tracer ', i, TRIM(Vname(1,idTvar(i)))
# endif
# if defined TANGENT || defined TL_IOMS
              WRITE (out,90) tl_Akt_fac(i,ng), 'tl_Akt_fac', i,         &
     &              'TLM basic state vertical mixing scale factor',     &
     &              'for tracer ', i, TRIM(Vname(1,idTvar(i)))
# endif
            END DO
#endif
            DO itrc=1,NBT
              i=idbio(itrc)
              WRITE (out,90) Tnudg(i,ng), 'Tnudg', i,                   &
     &              'Nudging/relaxation time scale (days)',             &
     &              'for tracer ', i, TRIM(Vname(1,idTvar(i)))
            END DO
#ifdef TCLIMATOLOGY
            DO itrc=1,NBT
              i=idbio(itrc)
              WRITE (out,100) LtracerCLM(i,ng), 'LtracerCLM',           &
     &              i, 'Processing climatology on tracer ', i,          &
     &              TRIM(Vname(1,idTvar(i)))
            END DO
#endif
#ifdef TS_PSOURCE
            DO itrc=1,NBT
              i=idbio(itrc)
              WRITE (out,100) LtracerSrc(i,ng), 'LtracerSrc',           &
     &              i, 'Processing point sources/Sink on tracer ', i,   &
     &              TRIM(Vname(1,idTvar(i)))
            END DO
#endif
            DO itrc=1,NBT
              i=idbio(itrc)
              IF (Hout(idTvar(i),ng)) WRITE (out,110)                   &
     &            Hout(idTvar(i),ng), 'Hout(idTvar)',                   &
     &            'Write out tracer ', i, TRIM(Vname(1,idTvar(i)))
            END DO
            DO itrc=1,NBT
              i=idbio(itrc)
              IF (Hout(idTsur(i),ng)) WRITE (out,110)                   &
     &            Hout(idTsur(i),ng), 'Hout(idTsur)',                   &
     &            'Write out tracer flux ', i, TRIM(Vname(1,idTvar(i)))
            END DO
#if defined AVERAGES    || \
   (defined AD_AVERAGES && defined ADJOINT) || \
   (defined RP_AVERAGES && defined TL_IOMS) || \
   (defined TL_AVERAGES && defined TANGENT)
            WRITE (out,'(1x)')
            DO itrc=1,NBT
              i=idbio(itrc)
              IF (Aout(idTvar(i),ng)) WRITE (out,110)                   &
     &            Aout(idTvar(i),ng), 'Aout(idTvar)',                   &
     &            'Write out averaged tracer ', i,                      &
     &            TRIM(Vname(1,idTvar(i)))
            END DO
#endif
#ifdef DIAGNOSTICS_TS
            WRITE (out,'(1x)')
            DO i=1,NBT
              itrc=idbio(i)
              IF (Dout(idDtrc(itrc,iTrate),ng))                         &
     &          WRITE (out,110) .TRUE., 'Dout(iTrate)',                 &
     &            'Write out rate of change of tracer ', itrc,          &
     &            TRIM(Vname(1,idTvar(itrc)))
            END DO
            DO i=1,NBT
              itrc=idbio(i)
              IF (Dout(idDtrc(itrc,iThadv),ng))                         &
     &          WRITE (out,110) .TRUE., 'Dout(iThadv)',                 &
     &            'Write out horizontal advection, tracer ', itrc,      &
     &            TRIM(Vname(1,idTvar(itrc)))
            END DO
            DO i=1,NBT
              itrc=idbio(i)
              IF (Dout(idDtrc(itrc,iTxadv),ng))                         &
     &          WRITE (out,110) .TRUE., 'Dout(iTxadv)',                 &
     &            'Write out horizontal X-advection, tracer ', itrc,    &
     &            TRIM(Vname(1,idTvar(itrc)))
            END DO
            DO i=1,NBT
              itrc=idbio(i)
              IF (Dout(idDtrc(itrc,iTyadv),ng))                         &
     &          WRITE (out,110) .TRUE., 'Dout(iTyadv)',                 &
     &            'Write out horizontal Y-advection, tracer ', itrc,    &
     &            TRIM(Vname(1,idTvar(itrc)))
            END DO
            DO i=1,NBT
              itrc=idbio(i)
              IF (Dout(idDtrc(itrc,iTvadv),ng))                         &
     &          WRITE (out,110) .TRUE., 'Dout(iTvadv)',                 &
     &            'Write out vertical advection, tracer ', itrc,        &
     &            TRIM(Vname(1,idTvar(itrc)))
            END DO
# if defined TS_DIF2 || defined TS_DIF4
            DO i=1,NBT
              itrc=idbio(i)
              IF (Dout(idDtrc(itrc,iThdif),ng))                         &
     &          WRITE (out,110) .TRUE., 'Dout(iThdif)',                 &
     &            'Write out horizontal diffusion, tracer ', itrc,      &
     &            TRIM(Vname(1,idTvar(itrc)))
            END DO
            DO i=1,NBT
              itrc=idbio(i)
              IF (Dout(idDtrc(i,iTxdif),ng))                            &
     &          WRITE (out,110) .TRUE., 'Dout(iTxdif)',                 &
     &            'Write out horizontal X-diffusion, tracer ', itrc,    &
     &            TRIM(Vname(1,idTvar(itrc)))
            END DO
            DO i=1,NBT
              itrc=idbio(i)
              IF (Dout(idDtrc(itrc,iTydif),ng))                         &
     &          WRITE (out,110) .TRUE., 'Dout(iTydif)',                 &
     &            'Write out horizontal Y-diffusion, tracer ', itrc,    &
     &            TRIM(Vname(1,idTvar(itrc)))
            END DO
#  if defined MIX_GEO_TS || defined MIX_ISO_TS
            DO i=1,NBT
              itrc=idbio(i)
              IF (Dout(idDtrc(itrc,iTsdif),ng))                         &
     &          WRITE (out,110) .TRUE., 'Dout(iTsdif)',                 &
     &            'Write out horizontal S-diffusion, tracer ', itrc,    &
     &            TRIM(Vname(1,idTvar(itrc)))
            END DO
#  endif
# endif
            DO i=1,NBT
              itrc=idbio(i)
              IF (Dout(idDtrc(itrc,iTvdif),ng))                         &
     &          WRITE (out,110) .TRUE., 'Dout(iTvdif)',                 &
     &            'Write out vertical diffusion, tracer ', itrc,        &
     &            TRIM(Vname(1,idTvar(itrc)))
            END DO
#endif
#ifdef DIAGNOSTICS_BIO
# if !(defined CARBON || defined OXYGEN || defined DENITRIFICATION)
            DO itrc=1,NDbio2d
              i=iDbio2(itrc)
              IF (Dout(i,ng)) WRITE (*,*)                           &
     &            Dout(i,ng), 'Hout(iDbio2)',                           &
     &            'Write out diagnostics for', TRIM(Vname(1,i))
            END DO
# endif
            WRITE(*,*) 'NDbio3d=',NDbio3d
            DO itrc=1,NDbio3d
              !WRITE(*,*) 'i=',i,'ng=',ng
              !WRITE(*,*)'iDbio3=',iDbio3,'Dout=',Dout(i,ng)
              i=iDbio3(itrc)
              IF (Dout(i,ng)) WRITE (*,*)                           &
     &            Dout(i,ng), 'Hout(iDbio3)',                           &
     &            'Write out diagnostics for', TRIM(Vname(1,i))
            END DO
#endif          
          END IF
        END DO
      END IF

!
!-----------------------------------------------------------------------
!  Rescale biological tracer parameters.
!-----------------------------------------------------------------------
!
!  Take the square root of the biharmonic coefficients so it can
!  be applied to each harmonic operator.
!
      DO ng=1,Ngrids
        DO itrc=1,NBT
          i=idbio(itrc)
          nl_tnu4(i,ng)=SQRT(ABS(nl_tnu4(i,ng)))
#ifdef ADJOINT
          ad_tnu4(i,ng)=SQRT(ABS(ad_tnu4(i,ng)))
#endif
#if defined TANGENT || defined TL_IOMS
          tl_tnu4(i,ng)=SQRT(ABS(tl_tnu4(i,ng)))
#endif
!
!  Compute inverse nudging coefficients (1/s) used in various tasks.
!
          IF (Tnudg(i,ng).gt.0.0_r8) THEN
            Tnudg(i,ng)=1.0_r8/(Tnudg(i,ng)*86400.0_r8)
          ELSE
            Tnudg(i,ng)=0.0_r8
          END IF
        END DO
      END DO

  30  FORMAT (/,' read_BioPar - variable info not yet loaded, ',        &
     &        a,i2.2,a)
  40  FORMAT (/,' read_BioPar - Error while processing line: ',/,a)
  50  FORMAT (/,/,' NPZD Model Parameters, Grid: ',i2.2,                &
     &        /,  ' ===============================',/)
  60  FORMAT (1x,i10,2x,a,t30,a)
  70  FORMAT (1p,e11.4,2x,a,t30,a)
  80  FORMAT (1p,e11.4,2x,a,t30,a,/,t32,a)
  90  FORMAT (1p,e11.4,2x,a,'(',i2.2,')',t30,a,/,t32,a,i2.2,':',1x,a)
 100  FORMAT (10x,l1,2x,a,'(',i2.2,')',t30,a,i2.2,':',1x,a)
 110  FORMAT (10x,l1,2x,a,t30,a,i2.2,':',1x,a)

!********************************************************EcoDynamo stuff*****************************************************************************      
! Initializing EcoDynamo objects

      DO ng=1,Ngrids 
         CALL light_new(LIGHTOBJ)
!	WRITE(*,*) "lightobj", LIGHTOBJ, LIGHTOBJM
     !Flagellates    
	 CALL   phytoplankton_new (PHYOBJ1,Pmax(ng),Iopt(ng),       &
     &                          ImaxE(ng),PhyIS(ng),                &
     &                         Beta(ng),                            &
     &                         AEiler(ng),BEiler(ng),               &
     &                         CEiler(ng),MaintResp(ng),            &
     &                         RespCoeff(ng),                       &
     &                         DocStressLoss(ng),PhyMRD(ng),        &
     &                         RedfieldC(ng),                       &
     &                         RedfieldN(ng),RedfieldP(ng),         &
     &                         TemperatureAugmentationRate(ng),     &
     &                         RatioLightDarkRespiration(ng),       &
     &                         minNPRatio(ng), maxNPRatio(ng),      &
     &                         PMaxUptake(ng),                      &
     &                         NMaxUptake(ng),KP(ng), K_NO3(ng),    &
     &                         KNH4(ng),                            &
     &                         MinPCellQuota(ng),MaxPCellQuota(ng), &
     &                         MinNCellQuota(ng),                   &
     &                         MaxNCellQuota(ng),                   &
     &                         KPInternal(ng), KNInternal(ng),      &
     &                         wPhy(ng),                            &
     &                         CarbonToOxygenProd(ng),              &
     &                         CarbonToOxygenResp(ng),              &
     &                         TminRespiration(ng),                 &
     &                         TminPhotosynthesis(ng),              &
     &                         NitrogenLimitation(ng),              &
     &                         PhosphorusLimitation(ng),0,          &
     &                         MaxSiCellQuota(ng),                  &
     &                         minNSiRatio(ng),                     &
     &                         SiMaxUptake(ng),                     &
     &                         KSi(ng),                             &
     &                         KSiInternal(ng),                     &
     &                         RedfieldSi(ng)                       )

         

        CALL dissobjt_new (DISSOBJ1, NitriR(ng), kdenit(ng),        &
     &                     knitO2(ng), kdenitO2(ng), kt(ng),        &
     &                     minRateN(ng),                            &
     &                     DenitNH4(ng),  &
     &                     MinToNH4(ng),I0(ng),KI(ng))
    
     !Diatoms
        CALL   phytoplankton_new (PHYOBJ2,PmaxDiatoms(ng),          &
     &                          IoptDiatoms(ng),                    &
     &                          ImaxEDiatoms(ng),PhyISDiatoms(ng),  &
     &                          BetaDiatoms(ng),                    &
     &                         AEilerDiatoms(ng),BEilerDiatoms(ng), &
     &                         CEilerDiatoms(ng),                   &
     &                         MaintRespDiatoms(ng),                &
     &                         RespCoeffDiatoms(ng),                &
     &                         DocStressLossDiatoms(ng),            &
     &                         PhyMRDDiatoms(ng),                   &
     &                         RedfieldCDiatoms(ng),                &
     &                         RedfieldNDiatoms(ng),                &
     &                         RedfieldPDiatoms(ng),                &
     &                  TemperatureAugmentationRateDiatoms(ng),     &
     &                         RatioLightDarkRespirationDiatoms(ng),&
     &                         minNPRatioDiatoms(ng),               &
     &                         maxNPRatioDiatoms(ng),               &
     &                         PMaxUptakeDiatoms(ng),               &
     &                         NMaxUptakeDiatoms(ng),KPDiatoms(ng), &
     &                         K_NO3Diatoms(ng),                    &
     &                         KNH4Diatoms(ng),                     &
     &                         MinPCellQuotaDiatoms(ng),            &
     &                         MaxPCellQuotaDiatoms(ng),            &
     &                         MinNCellQuotaDiatoms(ng),            &
     &                         MaxNCellQuotaDiatoms(ng),            &
     &                         KPInternalDiatoms(ng),               &
     &                         KNInternalDiatoms(ng),               &
     &                         wPhyDiatoms(ng),                     &
     &                         CarbonToOxygenProdDiatoms(ng),       &
     &                         CarbonToOxygenRespDiatoms(ng),       &
     &                         TminRespirationDiatoms(ng),          &
     &                         TminPhotosynthesisDiatoms(ng),       &
     &                         NitrogenLimitationDiatoms(ng),       &
     &                         PhosphorusLimitationDiatoms(ng),     &
     &                         SilicaLimitation(ng),                &
     &                         MaxSiCellQuota(ng),                  &
     &                         minNSiRatio(ng),                     &
     &                         SiMaxUptake(ng),                     &
     &                         KSi(ng),                             &
     &                         KSiInternal(ng),                     &
     &                         RedfieldSi(ng)                       )
      END DO
     !WRITE(*,*) "PHYOBJ1= ", PHYOBJ1,"PHYOBJ2= ", PHYOBJ2,'DISSOBJ1= ', DISSOBJ1
!****************************************************************************************************************************************************
      RETURN
      END SUBROUTINE read_BioPar
